#! /usr/bin/env python
"""Reads RST file from GSI
"""

import os
import re

# TODO add proper destructors


class SubMachine(object):
    """Define for each submachine."""

    def __init__(self, e_step, energy, focus_nr, focus_fl, subm_size):
        # print 'init submachine'
        self.energy_step = e_step
        self.energy = energy
        self.focus_step = focus_nr
        self.focus = focus_fl

        self.size = subm_size
        self.xpos = [0] * subm_size
        self.ypos = [0] * subm_size
        self.particles = [0] * subm_size

    def fillData(self, xx, yy, pp, n):
        # print "fill data:", n,  len(self.xpos)
        self.xpos[n] = xx
        self.ypos[n] = yy
        self.particles[n] = pp


class RstfileRead(object):
    """This class reads rst files."""

    FileIsRead = False

    def __init__(self, filename):
        """ Read the rst file."""
        # print 'initialized with filename',  filename

        if os.path.isfile(filename) is False:
            raise IOError("Could not find file " + filename)
        else:
            rstinput_file = open(filename, 'r')
            content = rstinput_file.readlines()
            # close(rstinput_file)
            data_length = len(content)
            print("read", data_length, "lines of data.")
            i = 0

            subm_counter = 0
            self.submachine = []
            while i < data_length:
                # print "parsing line",  i,  content[i]
                if re.match("sistable", content[i]) is not None:
                    self.sistable = content[i].split()[1]
                if re.match("rstfile", content[i]) is not None:
                    self.rstfile = content[i].split()[1]
                if re.match("patient_id", content[i]) is not None:
                    self.patient_id = content[i].split()[1]
                if re.match("projectile", content[i]) is not None:
                    self.projectile = content[i].split()[1]
                if re.match("charge", content[i]) is not None:
                    self.charge = int(content[i].split()[1])
                if re.match("mass", content[i]) is not None:
                    self.mass = int(content[i].split()[1])
                if re.match("gantryangle", content[i]) is not None:
                    self.gantryangle = float(content[i].split()[1])
                if re.match("couchangle", content[i]) is not None:
                    self.couchangle = float(content[i].split()[1])
                if re.match("stereotacticcoordinates", content[i]) is not None:
                    self.stereotactic = True
                else:
                    self.stereotactic = False
                if re.match("bolus", content[i]) is not None:
                    self.bolus = int(content[i].split()[1])
                if re.match("ripplefilter", content[i]) is not None:
                    self.ripplefilter = content[i].replace("ripplefilter ", "")
                if re.match("submachines", content[i]) is not None:
                    self.submachines = int(content[i].split()[1])

                if re.match("submachine#", content[i]) is not None:
                    # found a submachine, now do the data parsing
                    # print "found one submachine at",  i

                    # get an estimate of the submachine blocksize.
                    # It may be smaller, but certainly not larger than this.
                    i_stored = i
                    submachine_size = 0
                    i += 1
                    while (i < data_length) and (re.match("submachine#", content[i]) is None):
                        submachine_size += 1
                        i += 1
                    i = i_stored  # go back to the start of this submachine

                    data_counter = 0
                    self.submachine.append(
                        SubMachine(
                            int(content[i].split()[1]), float(content[i].split()[2]), int(content[i].split()[3]), float(
                                content[i].split()[4]), submachine_size))

                    i += 1
                    # now read the submachine
                    escape = False
                    while (i < data_length) and (re.match("submachine#", content[i]) is None) and (escape is False):
                        # print "subloop", i,  content[i]
                        if re.match("stepsize", content[i]) is not None:
                            self.submachine[subm_counter].stepsizex = int(content[i].split()[1])
                            self.submachine[subm_counter].stepsizey = int(content[i].split()[2])
                        else:
                            temp = content[i].split()
                            if len(temp) == 3:
                                try:
                                    x = float(temp[0])
                                    y = float(temp[1])
                                    z = float(temp[2])
                                    self.submachine[subm_counter].fillData(x, y, z, data_counter)
                                    data_counter += 1
                                except ValueError:
                                    # handles when some other comment is
                                    # inserted before the next "submachine"
                                    # line
                                    # print "escape", i, data_counter
                                    escape = True

                        i += 1
                    # number of entries in submachine
                    self.submachine[subm_counter].size = data_counter

                    # resize elements properly
                    # print "dc:", data_counter
                    self.submachine[subm_counter].xpos = self.submachine[subm_counter].xpos[:data_counter]

                    self.submachine[subm_counter].ypos = self.submachine[subm_counter].ypos[:data_counter]

                    self.submachine[subm_counter].particles = self.submachine[subm_counter].particles[:data_counter]
                    subm_counter += 1
                ################################
                else:
                    i += 1
            print("found", subm_counter, "submachines.")
            self.submachines = subm_counter

    def showVersion(self):
        print(self.version)


class SamfileRead(object):
    """This class reads rst files."""

    def __init__(self, filename):
        """ Read the rst file."""
        print('initialized with filename', filename)
        print("init")

    def __del__(self):
        object.__del__(self)
        print('deleted')

# example of usage:
# from rst_read import *
# a= RstfileRead("cub000101.rst")
# dir(a)
# print a.sistable
# print a.rstfile
# print a.couchangle
# print a.bolus
# print a.submachines
# b = a.submachine[0]



"""
    This file is part of PyTRiP.

    PyTRiP is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    PyTRiP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with PyTRiP.  If not, see <http://www.gnu.org/licenses/>
"""

import os

import numpy as np
from pytrip.error import InputError, ModuleNotLoadedError

from pytrip.cube import Cube

import pytrip as plib
import gc

try:
    from dicom.dataset import Dataset, FileDataset
    from dicom.sequence import Sequence

    _dicom_loaded = True
except:
    _dicom_loaded = False


def calculate_dose_cube(field, density_cube, isocenter, pre_dose, pathcube=None, factor=1.0):
    cube_size = [density_cube.pixel_size, density_cube.pixel_size, density_cube.slice_distance]
    basis = field.get_cube_basis()

    if pathcube is None:
        pathcube = plib.rhocube_to_water(np.array(density_cube.cube), np.array(basis[0]), np.array(cube_size))
        pathcube += field.bolus

    dist = plib.calculate_dist(pathcube, np.array(cube_size), isocenter, np.array(basis))
    # field_size = field.field_size # TODO why not used ?

    dist = np.reshape(dist, (density_cube.dimx * density_cube.dimy * density_cube.dimz, 3))
    raster_matrix = np.array(field.get_merged_raster_points())

    ddd = field.get_ddd_list()
    dose = plib.calculate_dose(dist, np.array(raster_matrix), np.array(ddd))

    dose = np.reshape(dose, (density_cube.dimz, density_cube.dimy, density_cube.dimx)) * 1.602189 * 10**(-8)
    dos = DosCube(density_cube)
    dos.cube = np.array(dose / pre_dose * 1000 * factor, dtype=np.int16)
    gc.collect()
    return dos, pathcube


class DosCube(Cube):

    data_file_extension = "dos"

    def __init__(self, cube=None):
        super(DosCube, self).__init__(cube)
        self.type = "DOS"
        self.target_dose = 0.0

    def read_dicom(self, dcm):
        if "rtdose" not in dcm:
            raise InputError("Data doesn't contain dose infomation")
        if self.header_set is False:
            self.read_dicom_header(dcm)
        self.cube = np.zeros((self.dimz, self.dimy, self.dimx))
        for i in range(len(dcm["rtdose"].pixel_array)):
            self.cube[i][:][:] = dcm["rtdose"].pixel_array[i]

    def calculate_dvh(self, voi):
        pos = 0
        size = np.array([self.pixel_size, self.pixel_size, self.slice_distance])
        dv = np.zeros(1500)
        valid = False
        for i in range(self.dimz):
            pos += self.slice_distance
            slice = voi.get_slice_at_pos(pos)
            if slice is not None:
                valid = True
                dv += plib.calculate_dvh_slice(self.cube[i], np.array(slice.contour[0].contour), size)
        if valid:
            cubes = sum(dv)
            dvh = np.cumsum(dv[::-1])[::-1] / cubes

            min_dose = np.where(dvh >= 0.98)[0][-1]
            max_dose = np.where(dvh <= 0.02)[0][0]
            area = cubes * size[0] * size[1] * size[2] / 1000
            mean = np.dot(dv, range(0, 1500)) / cubes
            return dvh, min_dose, max_dose, mean, area
        return None

    def create_dicom_plan(self):
        meta = Dataset()
        meta.MediaStorageSOPClassUID = '1.2.840.10008.5.1.4.1.1.2'
        meta.MediaStorageSOPInstanceUID = "1.2.3"
        meta.ImplementationClassUID = "1.2.3.4"
        ds = FileDataset("file", {}, file_meta=meta)
        ds.PatientsName = self.patient_name
        ds.PatientID = "123456"
        ds.PatientsSex = '0'
        ds.PatientsBirthDate = '19010101'
        ds.SpecificCharacterSet = 'ISO_IR 100'
        ds.SOPClassUID = '1.2.840.10008.5.1.4.1.1.2'
        ds.StudyInstanceUID = '1.2.3'
        ds.SOPInstanceUID = '1.2.3'

        ds.Modality = "RTPLAN"
        ds.SeriesDescription = 'RT Plan'
        ds.SeriesInstanceUID = '2.16.840.1.113662.2.12.0.3057.1241703565.43'
        ds.RTPlanDate = '19010101'
        ds.RTPlanGeometry = ''
        ds.RTPlanLabel = 'B1'
        ds.RTPlanTime = '000000'
        structure_ref = Dataset()
        structure_ref.RefdSOPClassUID = '1.2.840.10008.5.1.4.1.1.481.3'
        structure_ref.RefdSOPInstanceUID = '1.2.3'
        ds.RefdStructureSets = Sequence([structure_ref])

        dose_ref = Dataset()
        dose_ref.DoseReferenceNumber = 1
        dose_ref.DoseReferenceStructureType = 'SITE'
        dose_ref.DoseReferenceType = 'TARGET'
        dose_ref.TargetPrescriptionDose = self.target_dose
        dose_ref.DoseReferenceDescription = "TUMOR"
        ds.DoseReferences = Sequence([dose_ref])
        return ds

    def create_dicom(self):
        if not _dicom_loaded:
            raise ModuleNotLoadedError("Dicom")
        if not self.header_set:
            raise InputError("Header not loaded")

        ds = self.create_dicom_base()
        ds.Modality = 'RTDOSE'
        ds.SamplesperPixel = 1
        ds.BitsAllocated = self.num_bytes * 8
        ds.BitsStored = self.num_bytes * 8
        ds.AccessionNumber = ''
        ds.SeriesDescription = 'RT Dose'
        ds.DoseUnits = 'GY'
        ds.DoseType = 'PHYSICAL'
        ds.DoseGridScaling = self.target_dose / 10**5
        ds.DoseSummationType = 'PLAN'
        ds.SliceThickness = ''
        ds.InstanceCreationDate = '19010101'
        ds.InstanceCreationTime = '000000'
        ds.NumberOfFrames = len(self.cube)
        ds.PixelRepresentation = 0
        ds.StudyID = '1'
        ds.SeriesNumber = 14
        ds.GridFrameOffsetVector = [x * self.slice_distance for x in range(self.dimz)]
        ds.InstanceNumber = ''
        ds.NumberofFrames = len(self.cube)
        ds.PositionReferenceIndicator = "RF"
        ds.TissueHeterogeneityCorrection = ['IMAGE', 'ROI_OVERRIDE']
        ds.ImagePositionPatient = ["%.3f" % (self.xoffset * self.pixel_size), "%.3f" % (self.yoffset * self.pixel_size),
                                   "%.3f" % (self.slice_pos[0])]
        ds.SOPClassUID = '1.2.840.10008.5.1.4.1.1.481.2'
        ds.SOPInstanceUID = '1.2.246.352.71.7.320687012.47206.20090603085223'
        ds.SeriesInstanceUID = '1.2.246.352.71.2.320687012.28240.20090603082420'

        # Bind to rtplan
        rt_set = Dataset()
        rt_set.RefdSOPClassUID = '1.2.840.10008.5.1.4.1.1.481.5'
        rt_set.RefdSOPInstanceUID = '1.2.3'
        ds.ReferencedRTPlans = Sequence([rt_set])
        pixel_array = np.zeros((len(self.cube), ds.Rows, ds.Columns), dtype=self.pydata_type)
        pixel_array[:][:][:] = self.cube[:][:][:]
        ds.PixelData = pixel_array.tostring()
        return ds

    def write(self, path):
        f_split = os.path.splitext(path)
        header_file = f_split[0] + ".hed"
        dos_file = f_split[0] + ".dos"
        self.write_trip_header(header_file)
        self.write_trip_data(dos_file)

    def write_dicom(self, path):
        dcm = self.create_dicom()
        plan = self.create_dicom_plan()
        dcm.save_as(os.path.join(path, "rtdose.dcm"))
        plan.save_as(os.path.join(path, "rtplan.dcm"))



"""
    This file is part of PyTRiP.

    PyTRiP is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    PyTRiP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with PyTRiP.  If not, see <http://www.gnu.org/licenses/>
"""
import os

try:
    import dicom
except:
    pass


def compare_dicom_ct(dcm1, dcm2):
    diff = float(dcm1.ImagePositionPatient[2]) - float(dcm2.ImagePositionPatient[2])
    if diff > 0:
        return 1
    return -1


def read_dicom_folder(path):
    if not os.path.isdir(path):
        raise IOError("Folder does not exist")
    data = {}
    folder = os.listdir(path)
    for item in folder:
        if os.path.splitext(item)[1] == ".dcm":
            dcm = dicom.read_file(os.path.join(path, item), force=True)
            # TODO figureout what was it about (see below)
            # if dicom.__version__ >= "0.9.5":
            # dcm = dicom.read_file(os.path.join(path, item), force=True)
            # else:
            #     dcm = dicom.read_file(os.path.join(path, item))
            if dcm.Modality == "CT":
                if "images" not in data:
                    data["images"] = []
                data["images"].append(dcm)
            elif dcm.Modality == "RTSTRUCT":
                data["rtss"] = dcm
            elif dcm.Modality == "RTDOSE":
                data["rtdose"] = dcm
            elif dcm.Modality == "RTPLAN":
                data["rtplan"] = dcm
    if "images" in data:
        data["images"].sort(cmp=compare_dicom_ct)
    return data



#! /usr/bin/env python
"""
    converts gd files to xmgrace files in .agr format.
    06.10.2011 Armin Lühr
    ---
    comments:
    still limited to .gd files with a single y column
"""

import sys
import os


class ReadGd(object):
    """read file"""

    def __init__(self, filename):

        if os.path.isfile(filename) is False:
            raise IOError("Could not find file " + filename)

        gd_file = open(filename, 'r')
        gd_lines = gd_file.readlines()
        gd_file.close()

        header = "# Grace project file\n# \n@version 50122 \n"
        header += "@page size 842, 595 \n@page scroll 5% \n"
        header += "@page inout 5% \n@link page off \n"
        sys.stdout.write(header)

        line = gd_lines[0]
        line_len = len(line)
        string = '@    title  "' + line[:line_len - 1] + '"\n'
        string += '@    legend char size 1.250000\n'
        sys.stdout.write(string)

        count = -1

        for line in gd_lines[1:]:
            s = 0
            while s < 10:
                if line[s] == ' ':
                    s += 1
                else:
                    break

            if line[s].isdigit() or line[s] == '-':
                string = line

            elif line[s] == 'x' or line[s] == 'X':
                line_len = len(line)
                string = '@    xaxis  label "' + line[s + 2:line_len - 1]
                string += ' "\n@    xaxis  label char size 1.500000\n'
                string += '@    xaxis  ticklabel char size 1.250000\n'

            elif line[s] == 'y' or line[s] == 'Y':
                line_len = len(line)
                string = '@    yaxis  label "' + line[s + 2:line_len - 1]
                string += ' "\n@    yaxis  label char size 1.500000\n'
                string += '@    yaxis  ticklabel char size 1.250000\n'

            elif line[s] == 'h' \
                    or line[s] == 'H' \
                    or line[s] == 'a' \
                    or line[s] == 'A' \
                    or line[s] == 'n' \
                    or line[s] == 'N':
                count += 1
                string = '\n@    s' + str(count) + ' comment "' + filename + ' "\n'
                line_len = len(line)
                p = s + 1

                while p < line_len:
                    if line[p] == 'y' or line[p] == 'Y':
                        t = p + 1
                        while t < line_len:
                            if line[t] == ')':
                                string += '@    s' + str(count) + ' legend  "'
                                string += line[p + 2:t] + '"\n'
                                p = line_len
                                break
                            else:
                                t += 1
                    else:
                        p += 1

                string += '#' + line
            else:
                string = "#" + line

            sys.stdout.write(string)


if __name__ == '__main__':
    ReadGd(sys.argv[1])



#!/usr/bin/env python

import os
import sys

from pytrip import CtxCube, VdxCube


def main(args=sys.argv[1:]):
    print("\ttrip2dicom is a part of pytrip which was developed by \n\t"
          "Niels Bassler (bassler@phys.au.dk) and \n\t"
          "Jakob Toftegaard (jakob.toftegaard@gmail.com)")

    if len(args) != 2:
        print("\tusage: trip2dicom.py headerfile output_folder")
        print("\ttrip2dicom.py tripfile.hed dicomfolder/")
        exit()

    header_file_input_name = args[0]
    header_basename = os.path.splitext(header_file_input_name)[0]
    output_folder = args[1]

    _, data_file_name = CtxCube.parse_path(header_file_input_name)
    data_file_path = CtxCube.discover_file(data_file_name)

    if not os.path.exists(data_file_path):
        print("CTX file missing")
        exit()

    print("Convert CT images")
    c = CtxCube()
    c.read(header_basename)
    if not os.path.exists(output_folder):
        os.makedirs(output_folder)
    c.write_dicom(output_folder)

    if os.path.exists(header_basename + ".vdx"):
        print("Convert structures")
        v = VdxCube(c)
        v.read(header_basename + ".vdx")
        if not os.path.exists(output_folder):
            os.makedirs(output_folder)
        v.write_dicom(output_folder)
    print("Done")


if __name__ == '__main__':
    sys.exit(main(sys.argv[1:]))



"""
    This file is part of PyTRiP.

    PyTRiP is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    PyTRiP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with PyTRiP.  If not, see <http://www.gnu.org/licenses/>
"""


class DoseCube(object):
    def __init__(self, dosecube, type):
        self.dosecube = dosecube
        self.type = type

    def get_dosecube(self):
        return self.dosecube

    def get_type(self):
        return self.type

    def set_type(self, type):
        self.type = type

    def set_dose(self, value):
        self.dosecube.target_dose = float(value)

    def get_dose(self):
        return self.dosecube.target_dose

    def calculate_dvh(self, voi):
        dvh, min_dose, max_dose, mean, area = self.dosecube.calculate_dvh(voi)
        return dvh, min_dose, max_dose, mean, area



"""
    This file is part of PyTRiP.

    PyTRiP is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    PyTRiP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with PyTRiP.  If not, see <http://www.gnu.org/licenses/>
"""
# from pytrip.ctx import *
# from pytrip.dos import *
# from pytrip.vdx import *
# import pytrip.dicomhelper as dh
# from pytrip.res.point import *

# dcm = dh.read_dicom_folder(
# "../../../DicomData/Til Jacob T anonymiseret/Anonymiseret/")
# c = CtxCube()
# c.read_dicom(dcm)
#
# v = VdxCube("",c)
# v.read_dicom(dcm)



import unittest
import os
import tempfile

from tests.test_base import get_files

import pytrip.utils.trip2dicom
import pytrip.utils.dicom2trip


class TestTrip2Dicom(unittest.TestCase):
    def setUp(self):
        testdir = get_files()
        self.cube000 = os.path.join(testdir, "tst003000")

    def test_generate(self):
        # create temp dir
        tmpdir = tempfile.mkdtemp()

        # convert CT cube to DICOM
        pytrip.utils.trip2dicom.main([self.cube000, tmpdir])

        # check if destination directory is not empty
        self.assertTrue(os.listdir(tmpdir))


class TestDicom2Trip(unittest.TestCase):
    def test_check(self):
        self.assertRaises(SystemExit, pytrip.utils.dicom2trip.main, [])


if __name__ == '__main__':
    unittest.main()



"""
    This file is part of PyTRiP.

    PyTRiP is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    PyTRiP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with PyTRiP.  If not, see <http://www.gnu.org/licenses/>
"""
# from pytrip.tripexecuter.dosecube import DoseCube
# from pytrip.tripexecuter.field import Field
# from pytrip.tripexecuter.fieldcollection import FieldCollection
# from pytrip.tripexecuter.tripexecuter import TripExecuter
# from pytrip.tripexecuter.tripplan import TripPlan
# from pytrip.tripexecuter.tripplancollection import TripPlanCollection
# from pytrip.tripexecuter.tripvoi import TripVoi
# from pytrip.tripexecuter.voi import Voi
# from pytrip.tripexecuter.voicollection import VoiCollection
# from pytrip.tripexecuter.rbehandler import RBEHandler, RBE



"""
    This file is part of PyTRiP.

    PyTRiP is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    PyTRiP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with PyTRiP.  If not, see <http://www.gnu.org/licenses/>
"""
import os
import re
import sys
import logging

import numpy as np

try:
    import dicom
    from dicom.dataset import Dataset, FileDataset

    _dicom_loaded = True
except:
    _dicom_loaded = False

from pytrip.error import InputError, ModuleNotLoadedError

logger = logging.getLogger(__name__)


class Cube(object):
    def __init__(self, cube=None):
        if cube is not None:
            self.header_set = cube.header_set
            self.version = cube.version
            self.modality = cube.modality
            self.created_by = cube.created_by
            self.creation_info = cube.creation_info
            self.primary_view = cube.primary_view
            self.data_type = cube.data_type
            self.num_bytes = cube.num_bytes
            self.byte_order = cube.byte_order
            self.patient_name = cube.patient_name
            self.slice_dimension = cube.slice_dimension
            self.pixel_size = cube.pixel_size
            self.slice_distance = cube.slice_distance
            self.slice_number = cube.slice_number
            self.xoffset = cube.xoffset
            self.dimx = cube.dimx
            self.yoffset = cube.yoffset
            self.dimy = cube.dimy
            self.zoffset = cube.zoffset
            self.dimz = cube.dimz
            self.z_table = cube.z_table
            self.slice_pos = cube.slice_pos
            self.set_format_str()
            self.set_number_of_bytes()
            self.cube = np.zeros((self.dimz, self.dimy, self.dimx), dtype=cube.pydata_type)
        else:
            self.header_set = False
            self.version = "2.0"
            self.modality = "CT"
            self.created_by = "pytrip"
            self.creation_info = "create by pytrip"
            self.primary_view = "transversal"  # e.g. transversal
            self.data_type = ""
            self.num_bytes = ""
            self.byte_order = "vms"  # aix or vms
            self.patient_name = ""
            self.slice_dimension = ""  # eg. 256 meaning 256x256 pixels.
            self.pixel_size = ""  # size in mm
            self.slice_distance = ""  # thickness of slice
            self.slice_number = ""  # number of slices in file.
            self.xoffset = 0
            self.dimx = ""  # number of pixels along x (e.g. 256)
            self.yoffset = 0
            self.dimy = ""
            self.zoffset = 0
            self.dimz = ""
            self.slice_pos = []
            self.z_table = False  # list of slice#,pos(mm),thickness(mm),tilt

    def __add__(self, other):
        c = type(self)(self)
        if Cube in other.__class__.__bases__:
            c.cube = other.cube + self.cube
        else:
            c.cube = self.cube + float(other)
        return c

    def __sub__(self, other):
        c = type(self)(self)
        if Cube in other.__class__.__bases__:
            c.cube = self.cube - other.cube
        else:
            c.cube = self.cube - float(other)
        return c

    def __mul__(self, other):
        c = type(self)(self)
        if Cube in other.__class__.__bases__:
            c.cube = other.cube * self.cube
        else:
            t = type(c.cube[0, 0, 0])
            c.cube = np.array(self.cube * float(other), dtype=t)
        return c

    def __div__(self, other):
        c = type(self)(self)
        if Cube in other.__class__.__bases__:
            c.cube = self.cube / other.cube
        else:
            t = type(c.cube[0, 0, 0])
            c.cube = np.array(self.cube / float(other), dtype=t)
        c.cube[np.isnan(c.cube)] = 0  # fix division by zero NaNs
        return c

    def indices_to_pos(self, indices):
        pos = []
        pos.append((indices[0] + 0.5) * self.pixel_size + self.xoffset)
        pos.append((indices[1] + 0.5) * self.pixel_size + self.yoffset)
        pos.append(indices[2] * self.slice_distance + self.zoffset)
        return pos

    def slice_to_z(self, idx):
        return idx * self.slice_distance + self.zoffset

    def pos_to_indices(self, pos):
        indices = [int(pos[0] / self.pixel_size - self.xoffset / self.pixel_size),
                   int(pos[1] / self.pixel_size - self.yoffset / self.pixel_size),
                   int(pos[2] / self.slice_distance - self.zoffset / self.slice_distance)]
        return indices

    def get_value_at_indice(self, indices):
        return self.cube[indices[2]][indices[1]][indices[0]]

    def get_value_at_pos(self, pos):
        return self.get_value_at_indice(self.pos_to_indices(pos))

    def create_cube_from_equation(self, equation, center, limits, radial=True):
        # TODO why eq not being used ?
        # eq = util.evaluator(equation)
        # TODO why data not being used ?
        # data = np.array(np.zeros((self.dimz, self.dimy, self.dimx)))
        x = np.linspace(0.5, self.dimx - 0.5, self.dimx) * self.pixel_size - center[0]
        y = np.linspace(self.dimx - 0.5, 0.5, self.dimx) * self.pixel_size - center[1]
        xv, yv = np.meshgrid(x, y)

    def load_from_structure(self, voi, preset=0, data_type=np.int16):
        data = np.array(np.zeros((self.dimz, self.dimy, self.dimx)), dtype=data_type)
        if preset != 0:
            for i_z in range(self.dimz):
                for i_y in range(self.dimy):
                    intersection = voi.get_row_intersections(self.indices_to_pos([0, i_y, i_z]))
                    if intersection is None:
                        break
                    if len(intersection) > 0:
                        k = 0
                        for i_x in range(self.dimx):
                            if self.indices_to_pos([i_x, 0, 0])[0] > intersection[k]:
                                k += 1
                                if k >= (len(intersection)):
                                    break
                            if k % 2 == 1:
                                data[i_z][i_y][i_x] = preset
        self.cube = data

    def create_empty_cube(self, value, dimx, dimy, dimz, pixel_size, slice_distance):
        self.dimx = dimx
        self.dimy = dimy
        self.dimz = dimz
        self.slice_number = dimz
        self.pixel_size = pixel_size
        self.slice_distance = slice_distance
        self.cube = np.ones((dimz, dimy, dimx), dtype=np.int16) * (value)
        self.slice_dimension = dimz
        self.num_bytes = 2
        self.data_type = "integer"
        self.pydata_type = np.int16

    def override_cube_values(self, voi, value):
        for i_z in range(self.dimz):
            for i_y in range(self.dimy):
                intersection = voi.get_row_intersections(self.indices_to_pos([0, i_y, i_z]))
                if intersection is None:
                    break
                if len(intersection) > 0:
                    k = 0
                    for i_x in range(self.dimx):
                        if self.indices_to_pos([i_x, 0, 0])[0] > intersection[k]:
                            k += 1
                            if k >= (len(intersection)):
                                break
                        if k % 2 == 1:
                            self.cube[i_z][i_y][i_x] = value

    def set_offset_cube_values(self, voi, value):
        for i_z in range(self.dimz):
            for i_y in range(self.dimy):
                intersection = voi.get_row_intersections(self.indices_to_pos([0, i_y, i_z]))
                if intersection is None:
                    break
                if len(intersection) > 0:
                    k = 0
                    for i_x in range(self.dimx):
                        if self.indices_to_pos([i_x, 0, 0])[0] > intersection[k]:
                            k += 1
                            if k >= (len(intersection)):
                                break
                        if k % 2 == 1:
                            self.cube[i_z][i_y][i_x] += value

    def write_trip_header(self, path):
        output_str = "version " + self.version + "\n"
        output_str += "modality " + self.modality + "\n"
        output_str += "created_by " + self.created_by + "\n"
        output_str += "creation_info " + self.creation_info + "\n"
        output_str += "primary_view " + self.primary_view + "\n"
        output_str += "data_type " + self.data_type + "\n"
        output_str += "num_bytes " + str(self.num_bytes) + "\n"
        output_str += "byte_order " + self.byte_order + "\n"
        if self.patient_name == "":
            self.patient_name = "Anonyme"
        output_str += "patient_name " + self.patient_name + "\n"
        output_str += "slice_dimension " + str(self.slice_dimension) + "\n"
        output_str += "pixel_size " + str(self.pixel_size) + "\n"
        output_str += "slice_distance " + str(self.slice_distance) + "\n"
        output_str += "slice_number " + str(self.slice_number) + "\n"
        # output_str +=
        # "xoffset " + str(int(round(self.xoffset/self.pixel_size))) + "\n"
        output_str += "xoffset 0\n"
        output_str += "dimx " + str(self.dimx) + "\n"
        # output_str +=
        # "yoffset " + str(int(round(self.yoffset/self.pixel_size))) + "\n"
        output_str += "yoffset 0\n"
        output_str += "dimy " + str(self.dimy) + "\n"
        output_str += "zoffset 0\n"

        # """output_str += "zoffset " +
        # str(int(round(self.zoffset/self.slice_distance))) + "\n" """
        output_str += "dimz " + str(self.dimz) + "\n"
        output_str += "z_table no\n"

        # """if self.z_table is True:
        #     output_str += "z_table yes\n"
        #     output_str += "slice_no  position  thickness  gantry_tilt\n"
        #     for i in range(len(self.slice_pos)):
        #         output_str +=
        # "  %d\t%.4f\t%.4f\t%.4f\n"%(i+1,self.slice_pos[i],
        # self.slice_distance,0)"""
        with open(path, "w+") as f:
            f.write(output_str)

    def set_byteorder(self, endian=None):
        if endian is None:
            endian = sys.byteorder
        if endian == 'little':
            self.byte_order = "vms"
        elif endian == 'big':
            self.byte_order = "aix"
        else:
            print("HED error: unknown endian:", endian)
            sys.exit(-1)

    def set_format_str(self):
        if (self.byte_order == "vms"):
            self.format_str = "<"
        elif (self.byte_order == "aix"):
            self.format_str = ">"
        self.set_number_of_bytes()

    def set_number_of_bytes(self):
        if self.data_type == "integer":
            if self.num_bytes == 1:
                self.format_str += "b"
                self.pydata_type = np.int8
            if self.num_bytes == 2:
                self.format_str += "h"
                self.pydata_type = np.int16
            if self.num_bytes == 4:
                self.format_str += "i"
                self.pydata_type = np.int32
        elif self.data_type == "float":
            if self.num_bytes == 4:
                self.format_str += "f"
                self.pydata_type = np.float32
        elif self.data_type == "double":
            if self.num_bytes == 8:
                self.format_str += "d"
                self.pydata_type = np.double
        else:
            print("Format:", self.byte_order, self.data_type, self.num_bytes)
            raise IOError("Unsupported format.")

    def create_dicom_base(self):
        if _dicom_loaded is False:
            raise ModuleNotLoadedError("Dicom")
        if self.header_set is False:
            raise InputError("Header not loaded")
        meta = Dataset()
        meta.MediaStorageSOPClassUID = '1.2.840.10008.5.1.4.1.1.2'
        meta.MediaStorageSOPInstanceUID = "1.2.3"
        meta.ImplementationClassUID = "1.2.3.4"
        ds = FileDataset("file", {}, file_meta=meta)
        ds.PatientsName = self.patient_name
        ds.PatientID = "123456"
        ds.PatientsSex = '0'
        ds.PatientsBirthDate = '19010101'
        ds.SpecificCharacterSet = 'ISO_IR 100'
        ds.AccessionNumber = ''
        ds.is_little_endian = True
        ds.is_implicit_VR = True
        ds.file_meta.TransferSyntaxUID = dicom.UID.ImplicitVRLittleEndian
        ds.SOPClassUID = '1.2.3'  # !!!!!!!!
        ds.SOPInstanceUID = '1.2.3'  # !!!!!!!!!!
        ds.StudyInstanceUID = '1.2.3'  # !!!!!!!!!!
        ds.FrameofReferenceUID = '1.2.3'  # !!!!!!!!!
        ds.StudyDate = '19010101'  # !!!!!!!
        ds.StudyTime = '000000'  # !!!!!!!!!!
        ds.PhotometricInterpretation = 'MONOCHROME2'
        ds.SamplesPerPixel = 1
        ds.ImageOrientationPatient = ['1', '0', '0', '0', '1', '0']
        ds.Rows = self.dimx
        ds.Columns = self.dimy
        ds.SliceThickness = str(self.slice_distance)

        ds.PixelSpacing = [self.pixel_size, self.pixel_size]
        return ds

    def merge(self, cube):
        self.cube = np.maximum(self.cube, cube.cube)

    def merge_zero(self, cube):
        self.cube[self.cube == 0] = cube.cube[self.cube == 0]

    @staticmethod
    def discover_file(file_name):
        """
        Checks if file_name exists in the filesystem. If yes, gives back its path.
        If not, checks if gzipped file with same name exists.
        If gzipped file exists, gives back it path, otherwise returns None.
        :param file_name: File name or path
        :return: file_name, file_name + ".gz" or None
        """
        gzip_file_name = file_name + ".gz"
        if os.path.exists(file_name):
            logger.debug("File " + file_name + " exists")
            return file_name
        elif os.path.exists(gzip_file_name):
            logger.debug("File " + gzip_file_name + " exists")
            return gzip_file_name
        else:
            logger.error("Both " + file_name + " and " + gzip_file_name + " do not exists")
            return None

    @classmethod
    def parse_path(cls, path_name):
        """
        Parse path_name which can have form of: bare name (i.e. TST001), plain file (TST001.hed or TST001.ctx),
        gzipped file (TST001.hed.gz or TST001.ctx.gz) or some other name. Calculates plain file names of
        header and data file. Extension of data file is extracted from the class from which this method was called
        (.ctx for CtxCube, .dos for DosCube and LetCube). In case of non-parseable data, None is returned.

        >>> from pytrip import CtxCube
        >>> CtxCube.parse_path("frodo.hed")
        ('frodo.hed', 'frodo.ctx')
        >>> CtxCube.parse_path("baggins.ctx")
        ('baggins.hed', 'baggins.ctx')
        >>> CtxCube.parse_path("mordor")
        ('mordor.hed', 'mordor.ctx')
        >>> CtxCube.parse_path("legolas.hed.gz")
        ('legolas.hed', 'legolas.ctx')
        >>> CtxCube.parse_path("gimli.son.of.gloin.ctx.gz")
        ('gimli.son.of.gloin.hed', 'gimli.son.of.gloin.ctx')
        >>> CtxCube.parse_path("bilbo.dos")
        (None, None)
        >>> from pytrip import DosCube
        >>> DosCube.parse_path("bilbo.dos")
        ('bilbo.hed', 'bilbo.dos')
        >>> DosCube.parse_path("baggins.ctx")
        (None, None)

        :param path_name:
        :return:
        """

        logger.info("Parsing " + path_name)

        #  path can end with .gz or not (be archive or not)
        is_path_gzipped = path_name.endswith(".gz")

        logger.debug("is file gzipped ? " + str(is_path_gzipped))

        # strip .gz from path (if present)
        uncompressed_path_name = path_name
        if is_path_gzipped:
            uncompressed_path_name = os.path.splitext(path_name)[0]

        logger.debug("uncompressed filename: " + uncompressed_path_name)

        # now we have few possibilities
        #  1. file with known extension (i.e. TST001.hed or TST001.ctx)
        #  2. file without extension (i.e. TST001) or with unknown extension (i.e. TST001.mp3)
        # to distinguish these cases we will use has_path_known_extension flag

        # checking if current class has extension attribute, to compare with file extension
        if 'data_file_extension' in cls.__dict__:
            class_filename_extension = cls.data_file_extension.lower()
            logger.debug("class data filename extension: " + class_filename_extension)
        else:  # class without extension, we assume in such case file is given without extension
            logger.error("Class " + str(cls) + " doesn't have data_file_extension field")
            class_filename_extension = None

        # Case 1. - known extension, header file
        if uncompressed_path_name.endswith('.hed'):
            has_path_known_extension = True
            logger.debug("case 1, header file, has known extension ? " + str(has_path_known_extension))
        # Case 2. - file without extension
        elif not os.path.splitext(uncompressed_path_name)[1]:
            has_path_known_extension = False
            logger.debug("case 2, file without extension, has known extension ? " + str(has_path_known_extension))
        # Case 3. - file with extension, checking if known
        else:
            # splitting filename into core + extension parts
            core_name, core_ext = os.path.splitext(uncompressed_path_name)
            logger.debug("case 3, file with extension, core: " + core_name + " extension: " + core_ext)
            has_path_known_extension = (core_ext.lower() == "." + class_filename_extension)
            logger.debug("case 3, extension known ? " + str(has_path_known_extension))

        logger.debug("has file known extension ? " + str(has_path_known_extension))

        if has_path_known_extension:
            if uncompressed_path_name.endswith('.hed'):  # header file
                logger.debug("header file ")
                header_file = uncompressed_path_name
                cube_filename = os.path.splitext(uncompressed_path_name)[0] + "." + class_filename_extension
            else:  # cube file
                logger.debug("cube file ")
                header_file = os.path.splitext(uncompressed_path_name)[0] + ".hed"
                cube_filename = uncompressed_path_name
        else:
            if not os.path.splitext(uncompressed_path_name)[1]:  # file without extension
                logger.debug("file without extension ")
                header_file = uncompressed_path_name + ".hed"
                cube_filename = uncompressed_path_name + "." + class_filename_extension
            else:   # a problem
                logger.debug("a problem ")
                header_file, cube_filename = None, None

        return header_file, cube_filename

    def read_trip_header(self, content):
        i = 0
        self.header_set = True
        content = content.split('\n')
        has_ztable = False
        while i < len(content):
            if re.match("version", content[i]) is not None:
                self.version = content[i].split()[1]
            if re.match("modality", content[i]) is not None:
                self.modality = content[i].split()[1]
            if re.match("created_by", content[i]) is not None:
                self.created_by = content[i].replace("created_by ", "", 1)
                self.created_by = self.created_by.rstrip()
            if re.match("creation_info", content[i]) is not None:
                self.creation_info = content[i].replace("creation_info ", "", 1)
                self.creation_info = self.creation_info.rstrip()
            if re.match("primary_view", content[i]) is not None:
                self.primary_view = content[i].split()[1]
            if re.match("data_type", content[i]) is not None:
                self.data_type = content[i].split()[1]
            if re.match("num_bytes", content[i]) is not None:
                self.num_bytes = int(content[i].split()[1])
            if re.match("byte_order", content[i]) is not None:
                self.byte_order = content[i].split()[1]
            if re.match("patient_name", content[i]) is not None:
                self.patient_name = content[i].split()[1]
            if re.match("slice_dimension", content[i]) is not None:
                self.slice_dimension = int(content[i].split()[1])
            if re.match("pixel_size", content[i]) is not None:
                self.pixel_size = float(content[i].split()[1])
            if re.match("slice_distance", content[i]) is not None:
                self.slice_distance = float(content[i].split()[1])
            if re.match("slice_number", content[i]) is not None:
                self.slice_number = int(content[i].split()[1])
            if re.match("xoffset", content[i]) is not None:
                self.xoffset = int(content[i].split()[1])
            if re.match("yoffset", content[i]) is not None:
                self.yoffset = int(content[i].split()[1])
            if re.match("zoffset", content[i]) is not None:
                self.zoffset = int(content[i].split()[1])
            if re.match("dimx", content[i]) is not None:
                self.dimx = int(content[i].split()[1])
            if re.match("dimy", content[i]) is not None:
                self.dimy = int(content[i].split()[1])
            if re.match("dimz", content[i]) is not None:
                self.dimz = int(content[i].split()[1])
            if re.match("slice_no", content[i]) is not None:
                self.slice_pos = [float(j) for j in range(self.slice_number)]
                has_ztable = True
                i += 1
                for j in range(self.slice_number):
                    self.slice_pos[j] = float(content[i].split()[1])
                    i += 1
            i += 1
        self.zoffset *= self.slice_distance
        if has_ztable is not True:
            self.slice_pos = [float(j) for j in range(self.slice_number)]
            for i in range(self.slice_number):
                self.slice_pos[i] = self.zoffset + i * self.slice_distance
        self.set_format_str()
        self.set_number_of_bytes()

    def read(self, path):
        self.read_trip_data_file(path)

    def read_trip_header_file(self, path):
        """ Reads a header file, accepts also if suffix is missing, or if file"
        is in gz compressed. User can thus specify:
        tst001
        tst001.hed
        tst001.hed.gz

        and in any case it will attempt to load
        tst001.hed
        or
        tst001.hed.gz

        Note, first the un-zipped files will be attempted to read.
        Should these not exist, then the .gz are attempted.

        However, if the .hed.gz file was explicitly stated,
        then this file will also
        be loaded, even if a .hed is available.
        """

        # extract header name (blabla.hed) from path
        # we get blabla.hed even if path equals to blabla.hed.gz, see docs in Cube.parse_path function
        header_file_name, _ = self.parse_path(path)

        # figure out which file exists on disc: *.hed or *.hed.gz and get its path
        header_file_path = self.discover_file(header_file_name)

        # sanity check
        if header_file_path is not None:
            logger.info("Reading header file" + header_file_path)
        else:
            raise IOError("Could not find file " + path)

        # load plain of gzipped file
        if header_file_path.endswith(".gz"):
            import gzip
            fp = gzip.open(header_file_path, "rt")
        else:
            fp = open(header_file_path, "rt")
        content = fp.read()
        fp.close()

        # fill self with data
        self.read_trip_header(content)
        self.set_format_str()
        logger.debug("Format string:" + self.format_str)

    def read_trip_data_file(self, path, multiply_by_2=False):
        """
        Accepts path in similar way as read_trip_header_file.
        :param path:
        :param multiply_by_2:
        :return:
        """

        # extract header and data file name from path
        header_file_name, data_file_name = self.parse_path(path)

        # fill header data if self.header is empty
        if self.header_set is False:
            header_file_path = self.discover_file(header_file_name)

            # sanity check
            if header_file_path is not None:
                logger.info("Reading header file" + header_file_path)
                self.read_trip_header_file(header_file_path)
            else:
                raise IOError("Could not find file " + path)

        # raise exception if reading header failed
        if self.header_set is False:
            raise InputError("Header file not loaded")

        # preparation
        data_dtype = np.dtype(self.format_str)
        data_count = self.dimx * self.dimy * self.dimz

        # figure out path to data file on disk, might be gzipped or not
        data_file_path = self.discover_file(data_file_name)

        # load data from data file (gzipped or not)
        logger.info("Opening file: " + path)
        if data_file_path.endswith('.gz'):
            import gzip
            with gzip.open(data_file_path, "rb") as f:
                s = f.read(data_dtype.itemsize * data_count)
                cube = np.frombuffer(s, dtype=data_dtype, count=data_count)
        else:
            cube = np.fromfile(data_file_path, dtype=data_dtype)

        if self.byte_order == "aix":
            logger.info("AIX big-endian data.")
            # byteswapping is not needed anymore, handled by "<" ">" in dtype

        # sanity check
        logger.info("Cube data points : {:d}".format(len(cube)))
        if len(cube) != self.dimx * self.dimy * self.dimz:
            logger.error("Header size and cube size dont match.")
            logger.error("Cube data points : {:d}".format(len(cube)))
            logger.error("Header says      : {:d} = {:d} * {:d} * {:d}".format(
                self.dimx * self.dimy * self.dimz,
                self.dimx,
                self.dimy,
                self.dimz))
            raise IOError("Header data and dose cube size are not consistent.")

        cube = np.reshape(cube, (self.dimz, self.dimy, self.dimx))
        if multiply_by_2:
            logger.warning("Cube was rescaled to 50%. Multiplying with 2.")
            cube *= 2
        self.cube = cube

    def set_data_type(self, type):
        if type is np.int8 or type is np.uint8:
            self.data_type = "integer"
            self.num_bytes = 1
        elif type is np.int16 or type is np.uint16:
            self.data_type = "integer"
            self.num_bytes = 2
        elif type is np.int32 or type is np.uint32:
            self.data_type = "integer"
            self.num_bytes = 4
        elif type is np.float:
            self.data_type = "float"
            self.num_bytes = 4
        elif type is np.double:
            self.data_type = "double"
            self.num_bytes = 8

    def read_dicom_header(self, dcm):
        if _dicom_loaded is False:
            raise ModuleNotLoadedError("Dicom")
        ds = dcm["images"][0]
        self.version = "1.4"
        self.created_by = "pytrip"
        self.creation_info = "created by pytrip;"
        self.primary_view = "transversal"
        self.set_data_type(type(ds.pixel_array[0][0]))
        self.patient_name = ds.PatientsName
        self.slice_dimension = int(ds.Rows)  # should be changed ?
        self.pixel_size = float(ds.PixelSpacing[0])
        self.slice_distance = abs(
            float(dcm["images"][0].ImagePositionPatient[2]) - float(dcm["images"][1].ImagePositionPatient[2]))
        self.slice_number = len(dcm["images"])
        self.xoffset = float(ds.ImagePositionPatient[0])
        self.dimx = int(ds.Rows)
        self.yoffset = float(ds.ImagePositionPatient[1])
        self.dimy = int(ds.Columns)
        self.zoffset = float(ds.ImagePositionPatient[2])
        self.dimz = len(dcm["images"])
        self.z_table = True
        self.set_z_table(dcm)
        self.set_byteorder()
        self.set_format_str()
        self.header_set = True

    def set_z_table(self, dcm):
        self.slice_pos = []
        for i in range(len(dcm["images"])):
            self.slice_pos.append(float(dcm["images"][i].ImagePositionPatient[2]))

    def write_trip_data(self, path):
        cube = np.array(self.cube, dtype=self.pydata_type)
        if self.byte_order == "aix":
            cube = cube.byteswap()
        cube.tofile(path)
        return
        # TODO after return - probably forgotten
        # f = open(path, "wb+")
        # out = ""
        # _format = self.format_str[0] + self.format_str[1] * self.dimx
        # # i = 0 TODO why not used ?
        # for image in self.cube:
        #     out = ""
        #     for line in image:
        #         out += pack(_format, *line)
        #     f.write(out)
        # f.close()



"""
    This file is part of PyTRiP.

    PyTRiP is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    PyTRiP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with PyTRiP.  If not, see <http://www.gnu.org/licenses/>
"""

import os
import unittest

from pytrip.vdx import VdxCube

from pytrip.let import LETCube

import tests.test_base


class TestLet(unittest.TestCase):
    def setUp(self):
        testdir = tests.test_base.get_files()
        self.let001 = os.path.join(testdir, "tst003001.dosemlet.dos")
        self.vdx000 = os.path.join(testdir, "tst003000.vdx")

    def test_read(self):
        l = LETCube()
        l.read_trip_data_file(self.let001)
        v = VdxCube("")
        v.import_vdx(self.vdx000)
        # TODO fix loading VOI data
        # tumor = v.get_voi_by_name("Tumor Bed")
        # lvh = l.calculate_lvh(tumor)
        # plt.plot(lvh[0], lvh[1])
        # plt.show()


if __name__ == '__main__':
    unittest.main()



#! /usr/bin/env python

import sys
from optparse import OptionParser
from pylab import plt, ylabel, grid, xlabel, array
from pytrip.utils.rst_read import RstfileRead


def main(args=sys.argv[1:]):
    parser = OptionParser()
    parser.add_option("-s", "--submachine", dest="subm", help="Select submachine to plot.", metavar="int")
    parser.add_option(
        "-f", "--factor", dest="fac", help="Factor for scaling the blobs. "
        "Default is 1000.", metavar="int")
    (options, args) = parser.parse_args(args)

    file = args[0]

    sm = 1  # default
    fac = 1000
    if options.subm is not None:
        sm = int(options.subm)
    if options.fac is not None:
        fac = int(options.fac)

    a = RstfileRead(file)

    # convert data in submachine to a nice array
    b = a.submachine[sm]
    print("Submachine: ", sm, " - Energy:", b.energy, "MeV/u")
    cc = array(b.particles)

    cc = cc / cc.max() * fac

    fig = plt.figure()
    ax = fig.add_subplot(111)
    ax.scatter(b.xpos, b.ypos, c=cc, s=cc, alpha=0.75)
    ylabel("mm")
    xlabel("mm")

    grid(True)

    plt.show()


if __name__ == '__main__':
    sys.exit(main(sys.argv[1:]))



"""
    This file is part of PyTRiP.

    PyTRiP is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    PyTRiP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with PyTRiP.  If not, see <http://www.gnu.org/licenses/>
"""
import os
import unittest
import numpy as np

from pytrip.dos import DosCube
import pytriplib
import tests.test_base


class TestDos(unittest.TestCase):
    def setUp(self):
        testdir = tests.test_base.get_files()
        self.cube000 = os.path.join(testdir, "tst003001.dos")

    def test_read(self):
        c = DosCube()
        c.read(self.cube000)
        self.assertEqual(c.cube.shape[0], 300)
        self.assertEqual(c.cube.shape[1], 512)
        self.assertEqual(c.cube.shape[2], 512)

        # test method from C extension
        dose_center = pytriplib.calculate_dose_center(np.array(c.cube))
        self.assertEqual(dose_center.shape[0], 3)
        self.assertGreater(dose_center[0], 0.0)
        self.assertGreater(dose_center[1], 0.0)
        self.assertGreater(dose_center[2], 0.0)


if __name__ == '__main__':
    unittest.main()



"""
    This file is part of PyTRiP.

    PyTRiP is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    PyTRiP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with PyTRiP.  If not, see <http://www.gnu.org/licenses/>
"""
import os

import numpy as np

from pytrip.error import InputError
from pytrip.cube import Cube


class CtxCube(Cube):

    data_file_extension = "ctx"

    def __init__(self, cube=None):
        super(CtxCube, self).__init__(cube)
        self.type = "CTX"

    def read_dicom(self, dcm):
        if "images" not in dcm:
            raise InputError("Data doesn't contain ct data")
        if not self.header_set:
            self.read_dicom_header(dcm)

        self.cube = np.zeros((self.dimz, self.dimy, self.dimx), dtype=np.int16)
        intersect = float(dcm["images"][0].RescaleIntercept)
        slope = float(dcm["images"][0].RescaleSlope)

        for i in range(len(dcm["images"])):
            data = np.array(dcm["images"][i].pixel_array) * slope + intersect
            self.cube[i][:][:] = data
        if self.slice_pos[1] < self.slice_pos[0]:
            self.slice_pos.reverse()
            self.zoffset = self.slice_pos[0]
            self.cube = self.cube[::-1]

    def create_dicom(self):
        data = []

        for i in range(len(self.cube)):
            ds = self.create_dicom_base()
            ds.Modality = 'CT'
            ds.SamplesperPixel = 1
            ds.BitsAllocated = self.num_bytes * 8
            ds.BitsStored = self.num_bytes * 8
            ds.HighBit = self.num_bytes * 8 - 1
            ds.PatientPosition = 'HFS'
            ds.RescaleIntercept = 0.0
            ds.ImageType = ['ORIGINAL', 'PRIMARY', 'AXIAL']

            ds.PatientPosition = 'HFS'
            ds.SeriesInstanceUID = '2.16.840.1.113662.2.12.0.3057.1241703565.43'
            ds.RescaleSlope = 1.0
            ds.PixelRepresentation = 1
            ds.ImagePositionPatient = ["%.3f" % (self.xoffset * self.pixel_size),
                                       "%.3f" % (self.yoffset * self.pixel_size), "%.3f" % (self.slice_pos[i])]
            ds.SOPClassUID = '1.2.840.10008.5.1.4.1.1.2'
            ds.SOPInstanceUID = '2.16.1.113662.2.12.0.3057.1241703565.' + str(i + 1)

            ds.SeriesDate = '19010101'  # !!!!!!!!
            ds.ContentDate = '19010101'  # !!!!!!
            ds.SeriesTime = '000000'  # !!!!!!!!!
            ds.ContentTime = '000000'  # !!!!!!!!!

            ds.SliceLocation = str(self.slice_pos[i])
            ds.InstanceNumber = str(i + 1)
            pixel_array = np.zeros((ds.Rows, ds.Columns), dtype=self.pydata_type)
            pixel_array[:][:] = self.cube[i][:][:]
            ds.PixelData = pixel_array.tostring()
            ds.pixel_array = pixel_array
            data.append(ds)
        return data

    def write(self, path):
        f_split = os.path.splitext(path)
        header_file = f_split[0] + ".hed"
        ctx_file = f_split[0] + ".ctx"
        self.write_trip_header(header_file)
        self.write_trip_data(ctx_file)

    def write_dicom(self, path):
        dcm_list = self.create_dicom()
        for i in range(len(dcm_list)):
            dcm_list[i].save_as(os.path.join(path, "ct.%d.dcm" % (dcm_list[i].InstanceNumber - 1)))



"""
    This file is part of PyTRiP.

    PyTRiP is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    PyTRiP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with PyTRiP.  If not, see <http://www.gnu.org/licenses/>
"""
import gzip
import hashlib
import os
import tempfile
import unittest
import logging

from pytrip import DosCube
from pytrip.ctx import CtxCube
import tests.test_base

logger = logging.getLogger(__name__)


class TestCtx(unittest.TestCase):
    def setUp(self):
        testdir = tests.test_base.get_files()
        self.cube000 = os.path.join(testdir, "tst003000")

    def test_read(self):
        c = CtxCube()
        c.read(self.cube000)

    def read_and_write_cube(self, path):

        logger.info("Testing cube from path " + path)

        # read original cube and calculate hashsum
        c = CtxCube()
        c.read(path)

        _, data_file = CtxCube.parse_path(self.cube000)
        data_file_path = CtxCube.discover_file(data_file)

        if data_file_path.endswith(".gz"):
            f = gzip.open(data_file_path)
        else:
            f = open(data_file_path, 'rb')
        original_md5 = hashlib.md5(f.read()).hexdigest()
        f.close()

        # save cube and calculate hashsum
        fd, outfile = tempfile.mkstemp()
        c.write(outfile)
        f = open(outfile + ".ctx", 'rb')
        generated_md5 = hashlib.md5(f.read()).hexdigest()
        f.close()
        os.remove(outfile)
        # compare checksums
        self.assertEqual(original_md5, generated_md5)

    def test_write(self):
        possible_names = [self.cube000,
                          self.cube000 + ".ctx", self.cube000 + ".hed",
                          self.cube000 + ".hed.gz", self.cube000 + ".ctx.gz"]

        for name in possible_names:
            self.read_and_write_cube(name)

    def test_addition(self):
        # read cube
        c = CtxCube()
        c.read(self.cube000)
        d = c + 5
        self.assertEqual(c.cube[10][20][30] + 5, d.cube[10][20][30])

    def test_filename_parsing(self):
        bare_name = "frodo_baggins"
        cube_ext = ".ctx"
        header_ext = ".hed"
        gzip_ext = ".gz"

        # all possible reasonable names for input file
        testing_names = (bare_name,
                         bare_name + cube_ext, bare_name + cube_ext + gzip_ext,
                         bare_name + header_ext, bare_name + header_ext + gzip_ext)

        # loop over all names
        for name in testing_names:
            logger.info("Parsing " + name)
            header_filename, cube_filename = CtxCube.parse_path(name)

            self.assertIsNotNone(header_filename)
            self.assertIsNotNone(cube_filename)

            logger.info("Parsing output: " + header_filename + " , " + cube_filename)

            # test if got what was expected
            self.assertEqual(header_filename, bare_name + header_ext)
            self.assertEqual(cube_filename, bare_name + cube_ext)

        dos_cube_ext = ".dos"

        # all possible reasonable names for input file
        testing_names = (bare_name,
                         bare_name + dos_cube_ext, bare_name + dos_cube_ext + gzip_ext,
                         bare_name + header_ext, bare_name + header_ext + gzip_ext)

        # loop over all names
        for name in testing_names:
            logger.info("Parsing " + name)
            dos_header_filename, dos_cube_filename = DosCube.parse_path(name)

            self.assertIsNotNone(dos_header_filename)
            self.assertIsNotNone(dos_cube_filename)

            logger.info("Parsing output: " + dos_header_filename + " , " + dos_cube_filename)

            # test if got what was expected
            self.assertEqual(dos_header_filename, bare_name + header_ext)
            self.assertEqual(dos_cube_filename, bare_name + dos_cube_ext)


if __name__ == '__main__':
    unittest.main()



from pytrip.tripexecuter.pytripobj import pytripObj
from pytrip.tripexecuter.fieldcollection import FieldCollection
from pytrip.tripexecuter.voicollection import VoiCollection
from pytrip.tripexecuter.tripexecuter import TripExecuter, InputError
from pytrip.dos import DosCube
from pytrip.let import LETCube
from pytrip.tripexecuter.dosecube import DoseCube
import os


class TripPlan(pytripObj):
    def __init__(self, name="", comment=""):
        self.save_fields = ["fields", "vois", "remote", "servername", "username", "password", "working_dir",
                            "ddd_folder", "spc_folder", "sis_file", "res_tissue_type", "target_tissue_type", "name",
                            "comment"]
        self.save_fields.extend(["iterations", "opt_method", "opt_princip", "dose_alg", "bio_alg", "opt_alg", "eps",
                                 "geps", "out_phys_dose", "out_bio_dose", "out_dose_mean_let", "out_field",
                                 "out_mean_let", "dose_percent"])
        self.fields = FieldCollection(self)
        self.vois = VoiCollection(self)
        self.remote = False
        self.servername = ""
        self.username = ""
        self.password = ""
        self.working_dir = "$HOME/tmp"
        self.ddd_folder = ""
        self.spc_folder = ""
        self.sis_file = ""
        self.res_tissue_type = ""
        self.target_tissue_type = ""
        self.active = False
        self.name = name
        self.comment = comment
        self.dosecube = None
        self.dosecubes = []
        self.optimize = True
        self.letcube = None
        self.iterations = 500
        self.opt_method = "Phys"
        self.opt_princip = "H2OBased"
        self.dose_alg = "classic"
        self.bio_alg = "classic"
        self.opt_alg = "classic"
        self.eps = 1e-3
        self.geps = 1e-4

        self.out_phys_dose = True
        self.out_bio_dose = False
        self.out_dose_mean_let = False
        self.out_field = False
        self.out_mean_let = False
        self.window = []
        self.dose_percent = {}
        self.target_dose_cube = None

    def Init(self, parent):
        self.parent = parent
        self.vois.Init()
        self.fields.Init()

    def set_optimize(self, optimize):
        self.optimize = optimize

    def get_optimize(self):
        return self.optimize

    def set_window(self, window):
        self.window = window

    def get_window(self):
        return self.window

    def set_target_dose_cube(self, dos):
        self.target_dose_cube = dos

    def get_target_dose_cube(self):
        return self.target_dose_cube

    def set_dose_percent(self, ion, dose_percent):
        if dose_percent == "":
            del self.dose_percent[ion]
        self.dose_percent[ion] = float(dose_percent)

    def get_dose_percent(self, ion):
        if ion in self.dose_percent:
            return self.dose_percent[ion]
        return None

    def get_all_dose_percent(self):
        return self.dose_percent

    def remove_dose_percent(self, ion):
        del self.dose_percent[ion]

    def save_plan(self, images, path):
        self.save_exec(images, path + ".exec")
        self.save_data(images, path + ".exec")

    def save_exec(self, images, path):
        t = TripExecuter(images)
        t.set_plan(self)
        t.save_exec(path)

    def save_data(self, images, path):
        t = TripExecuter(images)
        t.set_plan(self)
        t.save_data(path)
        for dos in self.dosecubes:
            dos.get_dosecube().write(path + "." + dos.get_type() + ".dos")
        if self.letcube is not None:
            self.letcube.write(path + "." + "dosmeanlet.dos")

    def get_projectile(self):
        if len(self.fields):
            return self.fields.get_fields()[0].get_projectile()

    def get_out_phys_dose(self):
        return self.out_phys_dose

    def set_out_phys_dose(self, value):
        self.out_phys_dose = value

    def get_out_bio_dose(self):
        return self.out_bio_dose

    def set_out_bio_dose(self, value):
        self.out_bio_dose = value

    def get_out_dose_mean_let(self):
        return self.out_dose_mean_let

    def set_out_dose_mean_let(self, value):
        self.out_dose_mean_let = value

    def get_out_mean_let(self):
        return self.out_dose_mean_let

    def set_out_mean_let(self, value):
        self.out_dose_mean_let = value

    def get_out_field(self):
        return self.out_field

    def set_out_field(self, value):
        self.out_field = value

    def set_res_tissue_type(self, res_tissue_type):
        self.res_tissue_type = res_tissue_type

    def get_res_tissue_type(self):
        return self.res_tissue_type

    def set_target_tissue_type(self, target_tissue_type):
        self.target_tissue_type = target_tissue_type

    def get_target_tissue_type(self):
        return self.target_tissue_type

    def is_remote(self):
        return self.remote

    def set_remote_state(self, state):
        self.remote = state

    def get_server(self):
        return self.servername

    def set_server(self, server):
        self.servername = server

    def get_username(self):
        return self.username

    def set_username(self, username):
        self.username = username

    def get_password(self):
        return self.password

    def set_password(self, password):
        self.password = password

    def get_working_dir(self):
        return self.working_dir

    def set_working_dir(self, working_dir):
        self.working_dir = working_dir

    def get_iterations(self):
        return self.iterations

    def get_eps(self):
        return self.eps

    def get_geps(self):
        return self.geps

    def get_bio_algorithm(self):
        return self.bio_alg

    def get_opt_algorithm(self):
        return self.opt_alg

    def get_opt_method(self):
        return self.opt_method

    def set_opt_method(self, method):
        self.opt_method = method

    def get_dose_algorithm(self):
        return self.dose_alg

    def active_dose_change(self, dos):
        self.dosecube = dos

    def load_let(self, path):
        if hasattr(self, "letcube"):
            if self.letcube is not None:
                self.remove_let(self.letcube)
        let = LETCube()
        let.read(os.path.splitext(path)[0] + ".dos")

        self.letcube = let
        return let

    def remove_let(self, let):
        self.letcube = None

    def get_let(self):
        if hasattr(self, "letcube"):
            return self.letcube
        return None

    def get_let_cube(self):
        if self.letcube is not None:
            return self.letcube.cube
        return None

    def get_dose(self, type=""):
        if len(self.dosecubes):
            if type == "":
                return self.dosecube
            for cube in self.dosecubes:
                if cube.get_type() == type:
                    return cube
        return None

    def get_dose_cube(self):
        if self.dosecube is not None:
            return self.dosecube.get_dosecube().cube
        return None

    def remove_dose(self, dos):
        self.dosecubes.remove(dos)
        if self.dosecube is dos:
            self.dosecube = None
        self.clean_cache()

    def remove_dose_by_type(self, type):
        dos = None
        for cube in self.dosecubes:
            if cube.get_type() == type:
                dos = cube
        if dos is not None:
            self.dosecubes.remove(dos)
            self.clean_cache()
        return dos

    def clean_cache(self):
        for voi in self.vois:
            voi.clean_cache()

    def add_dose(self, dos, t=""):
        if type(dos) is DosCube:
            dos = DoseCube(dos, t)
        if hasattr(self, "dosecube"):
            if self.dosecube is not None:
                self.remove_dose_by_type(dos.get_type())
        self.dosecubes.append(dos)

        self.set_active_dose(dos)

    def set_active_dose(self, dos):
        self.dosecube = dos
        self.clean_cache()

    def load_dose(self, path, t, target_dose=0.0):
        dos = DosCube()
        dos.read(os.path.splitext(path)[0] + ".dos")
        d = DoseCube(dos, t)
        d.set_dose(target_dose)
        self.add_dose(d)

    def set_eps(self, value):
        try:
            value = float(value)
            if value < 0:
                raise Exception()
            self.eps = value
        except Exception as e:
            raise InputError("eps shoud be a number Larger than 0" + e)

    def set_geps(self, value):
        try:
            value = float(value)
            if value < 0:
                raise Exception()
            self.geps = value

        except Exception as e:
            raise InputError("geps shoud be a number Larger than 0" + e)

    def set_iterations(self, value):
        try:
            value = int(value)
            if value < 0:
                raise Exception()
            self.iterations = value
        except Exception as e:
            raise InputError("iterations shoud be a number Larger than 0" + e)

    def set_bio_algorithm(self, value):
        self.bio_alg = value

    def set_dose_algorithm(self, value):
        self.dose_alg = value

    def set_opt_algorithm(self, value):
        self.opt_alg = value

    def get_opt_princip(self):
        return self.opt_princip

    def set_opt_princip(self, value):
        self.opt_princip = value

    def set_phys_bio(self, value):
        self.phys_bio = value

    def set_name(self, name):
        if self.parent.get_plan(name) is None:
            self.name = name
            return True
        return False

    def get_name(self):
        return self.name

    def get_vois(self):
        return self.vois

    def get_fields(self):
        return self.fields

    def add_voi(self, voi):
        return self.vois.add_voi(voi)

    def add_field(self, field):
        return self.fields.add_field(field)

    def remove_field(self, field):
        self.fields.remove(field)

    def get_field_datasource(self):
        return self.fields

    def get_voi_datasource(self):
        return self.vois

    def destroy(self):
        self.vois.destroy()
        self.fields.destroy()

    def set_ddd_folder(self, path):
        self.ddd_folder = path

    def get_ddd_folder(self):
        return self.ddd_folder

    def get_sis_file(self):
        return self.sis_file

    def set_sis_file(self, path):
        self.sis_file = path

    def set_spc_folder(self, path):
        self.spc_folder = path

    def get_spc_folder(self):
        return self.spc_folder



"""
    This file is part of PyTRiP.

    PyTRiP is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    PyTRiP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with PyTRiP.  If not, see <http://www.gnu.org/licenses/>
"""

import os

import numpy as np
from pytrip.cube import Cube

import pytrip as plib


class LETCube(Cube):

    data_file_extension = "dos"

    def __init__(self, cube=None):
        super(LETCube, self).__init__(cube)

    def write(self, path):
        f_split = os.path.splitext(path)
        header_file = f_split[0] + ".dosemlet.hed"
        dos_file = f_split[0] + ".dosemlet.dos"
        self.write_trip_header(header_file)
        self.write_trip_data(dos_file)

    def get_max(self):
        return np.amax(self.cube)

    def calculate_lvh(self, voi):
        pos = 0
        size = np.array([self.pixel_size, self.pixel_size, self.slice_distance])
        lv = np.zeros(3000)
        for i in range(self.dimz):
            pos += self.slice_distance
            slice = voi.get_slice_at_pos(pos)
            if slice is not None:
                lv += plib.calculate_lvh_slice(self.cube[i], np.array(slice.contour[0].contour), size)

        cubes = sum(lv)
        lvh = np.cumsum(lv[::-1])[::-1] / cubes
        min_let = np.where(lvh >= 0.98)[0][-1]
        max_let = np.where(lvh <= 0.02)[0][0]
        area = cubes * size[0] * size[1] * size[2] / 1000
        mean = np.dot(lv, range(0, 3000)) / cubes
        return lvh, min_let, max_let, mean, area

    def write_lvh_to_file(self, voi, path):
        lvh = self.calculate_lvh(voi)
        output = ""
        for vol, let in zip(lvh[0], lvh[1]):
            output += "%.4e\t%.4e\n" % (vol, let)
        f = open(path + ".dvh", "w+")
        f.write(output)
        f.close()



"""
    This file is part of PyTRiP.

    PyTRiP is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    PyTRiP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with PyTRiP.  If not, see <http://www.gnu.org/licenses/>
"""

import numpy as np

try:
    import matplotlib.pyplot as plt
    import matplotlib
    import matplotlib._cntr as cntr
except:
    pass

import pytrip.res.point


class PlotUtil:
    def __init__(self):
        matplotlib.interactive(True)
        self.contrast = [-100, 400]
        self.vois = []
        self.plot_vois = True
        self.dose_plot = "colorwash"
        self.dosecontour_levels = []
        self.let_plot = "colorwash"
        self.dose_axis = "auto"
        self.colormap_dose = plt.get_cmap(None)
        self.colormap_let = plt.get_cmap(None)
        self.fields = []
        self.figure = plt
        self.draw_in_gui = False
        self.voi_plots = []
        self.plot_plan = "Transversal"
        self.draw_text = True
        self.factor = 10
        self.zoom = 100.0
        self.center = [50.0, 50.0]
        self.plan = None

    def add_dose_contour(self, dose_contour):
        self.dosecontour_levels.append(dose_contour)

    def get_dose_contours(self):
        return self.dosecontour_levels

    def remove_dose_contour(self, dose_contour):
        self.dosecontour_levels.remove(dose_contour)

    def set_plan(self, plan):
        self.plan = plan

    def get_colorbar(self):
        return self.dose_bar

    def set_zoom(self, zoom):
        if zoom < self.zoom:
            self.zoom = zoom
            offset = self.get_offset()
            size = self.get_size()
            width = float(size[0]) / self.zoom * 100.0
            height = float(size[1]) / self.zoom * 100.0
            # TODO center never used, why ?
            # center = [float(size[0]) * self.center[0] / 100,
            #           float(size[1]) * self.center[1] / 100]
            if offset[0] < 0:
                self.center[0] += (-offset[0]) * 100 / float(size[0])
            elif offset[0] + width > size[0]:
                self.center[0] -= (offset[0] + width - size[0]) * 100 / float(size[0])

            if offset[1] < 0:
                self.center[1] += (-offset[1]) * 100 / float(size[1])
            elif offset[1] + height > size[1]:
                self.center[1] -= (offset[1] + height - size[1]) * 100 / float(size[1])
        else:
            self.zoom = zoom

    def get_offset(self):
        size = self.get_size()
        width = float(size[0]) / self.zoom * 100.0
        height = float(size[1]) / self.zoom * 100.0
        center = [float(size[0]) * self.center[0] / 100, float(size[1]) * self.center[1] / 100]
        offset = [center[0] - width / 2, center[1] - height / 2]
        return offset

    def move_center(self, offset):
        if self.zoom == 100:
            return False

        size = self.get_size()
        width = float(size[0]) / self.zoom * 100.0
        height = float(size[1]) / self.zoom * 100.0
        scale_factor = self.fig_ct.get_figure().get_size_inches()[0] * self.fig_ct.get_figure().get_dpi() / width / 2

        center = [float(size[0]) * self.center[0] / 100, float(size[1]) * self.center[1] / 100]
        center[0] -= float(offset[0]) / scale_factor
        center[1] += float(offset[1]) / scale_factor / self.aspect

        off = [center[0] - width / 2, center[1] - height / 2]

        if off[0] > 0 and off[0] + width < size[0]:
            self.center[0] = center[0] / float(size[0]) * 100
        if off[1] > 0 and off[1] + height < size[1]:
            self.center[1] = center[1] / float(size[1]) * 100
        return True

    def get_zoom(self):
        return self.zoom

    def get_images_count(self):
        if self.plot_plan == "Transversal":
            return len(self.ctx.cube)
        elif self.plot_plan == "Sagital":
            return len(self.ctx.cube[0, 0])
        elif self.plot_plan == "Coronal":
            return len(self.ctx.cube[0])

    def set_draw_in_gui(self, yes):
        self.draw_in_gui = yes

    def set_plot_plan(self, plot_plan):
        self.plot_plan = plot_plan
        self.clear()

    def clear(self):
        if hasattr(self, "fig_dose"):
            del self.fig_dose
        if hasattr(self, "dose_bar"):
            del self.dose_bar
        if hasattr(self, "contrast_bar"):
            del self.contrast_bar
        if hasattr(self, "let_bar"):
            del self.let_bar
        if hasattr(self, "fig_ct"):
            del self.fig_ct

    def set_colormap_dose(self, colormap):
        self.colormap_dose = colormap
        self.clear_dose_view()

    def set_figure(self, figure):
        self.figure = figure

    def set_dose_axis(self, dose_axis):
        self.dose_axis = dose_axis
        self.clear_dose_view()

    def set_colormap_let(self, colormap):
        self.colormap_let = colormap

    def set_ct(self, ctx):
        self.ctx = ctx

    def set_let(self, let):
        self.let = let
        if self.let is not None:
            self.max_let = np.amax(let.cube)
            self.min_let = 0

    def set_dose_plot(self, type):
        self.dose_plot = type
        self.clear_dose_view()

    def get_dose_plot(self):
        return self.dose_plot

    def set_dose(self, dos):
        self.dos = dos
        if self.dos is not None:
            self.max_dose = np.amax(dos.cube) / self.factor
            self.min_dose = 0
            if hasattr(self, "fig_dose"):
                self.fig_dose.set_clim(vmin=self.min_dose, vmax=self.max_dose)

    def set_dose_min_max(self, min_max):
        if min_max[0] >= 0 and min_max[0] < min_max[1]:
            self.min_dose = min_max[0]
            self.max_dose = min_max[1]
            if hasattr(self, "fig_dose"):
                self.fig_dose.set_clim(vmin=self.min_dose, vmax=self.max_dose)

    def set_let_min_max(self, min_max):
        if min_max[0] >= 0 and min_max[0] < min_max[1]:
            self.min_let = min_max[0]
            self.max_let = min_max[1]
            if hasattr(self, "fig_let"):
                self.fig_let.set_clim(vmin=self.min_let, vmax=self.max_let)

    def get_min_max_dose(self):
        return [self.min_dose, self.max_dose]

    def get_min_max_let(self):
        return [self.min_let, self.max_let]

    def add_voi(self, voi):
        self.vois.append(voi)

    def remove_voi(self, voi):
        self.vois.remove(voi)

    def set_contrast(self, contrast):
        if contrast[0] >= contrast[1] or contrast[1] > 2000 or contrast[0] < -1000:
            return
        self.contrast = contrast
        if hasattr(self, "fig_ct"):
            self.fig_ct.set_clim(vmin=contrast[0], vmax=contrast[1])

    def pixel_to_pos(self, pixel):
        if self.plot_plan == "Transversal":
            pos = [pixel[0] * self.ctx.pixel_size, pixel[1] * self.ctx.pixel_size]
        elif self.plot_plan == "Sagital":
            pos = [(self.ctx.dimy - pixel[0]) * self.ctx.pixel_size,
                   (self.ctx.dimz - pixel[1]) * self.ctx.slice_distance]
        elif self.plot_plan == "Coronal":
            pos = [pixel[0] * self.ctx.pixel_size, (self.ctx.dimz - pixel[1]) * self.ctx.slice_distance]
        return [pixel, pos]

    def get_contrast(self):
        return self.contrast

    def get_dose(self):
        if hasattr(self, "dos"):
            return self.dos
        return None

    def get_size(self):
        if self.plot_plan == "Transversal":
            width = self.ctx.dimx
            height = self.ctx.dimy
        elif self.plot_plan == "Sagital":
            width = self.ctx.dimy
            height = self.ctx.dimz
        elif self.plot_plan == "Coronal":
            width = self.ctx.dimx
            height = self.ctx.dimz
        return [width, height]

    def get_let(self):
        if hasattr(self, "let"):
            return self.let
        return None

    def plot(self, idx):
        if self.plot_plan == "Transversal":
            ct_data = self.ctx.cube[idx]
            self.aspect = 1.0
        elif self.plot_plan == "Sagital":
            ct_data = self.ctx.cube[-1:0:-1, -1:0:-1, idx]
            self.aspect = self.ctx.slice_distance / self.ctx.pixel_size
        elif self.plot_plan == "Coronal":
            ct_data = self.ctx.cube[-1:0:-1, idx, -1:0:-1]
            self.aspect = self.ctx.slice_distance / self.ctx.pixel_size

        if not hasattr(self, "fig_ct"):
            self.fig_ct = self.figure.imshow(
                ct_data, cmap=plt.get_cmap("gray"), vmin=self.contrast[0], vmax=self.contrast[1], aspect=self.aspect)
        else:
            self.fig_ct.set_data(ct_data)

        if self.plot_plan == "Transversal":
            self.figure.axis([0, self.ctx.dimx, self.ctx.dimy, 0])
        elif self.plot_plan == "Sagital":
            self.figure.axis([0, self.ctx.dimy, self.ctx.dimz, 0])
        elif self.plot_plan == "Coronal":
            self.figure.axis([0, self.ctx.dimx, self.ctx.dimz, 0])
        if self.draw_in_gui:
            self.figure.axes.get_xaxis().set_visible(False)
            self.figure.axes.get_yaxis().set_visible(False)
            if not hasattr(self, "contrast_bar"):
                cax = self.figure.figure.add_axes([0.1, 0.1, 0.03, 0.8])
                self.contrast_bar = self.figure.figure.colorbar(self.fig_ct, cax=cax)
        # ~
        self.clean_plot()
        self.plot_dose(idx)
        self.plot_let(idx)

        if self.plot_vois:
            for i, voi in enumerate(self.vois):
                plot = False
                data = []
                if self.plot_plan == "Transversal":
                    slice = voi.get_slice_at_pos(self.ctx.slice_to_z(idx))
                    if slice is None:
                        continue
                    for contour in slice.contour:
                        data.append(np.array(contour.contour))
                        plot = True
                elif self.plot_plan == "Sagital":
                    slice = voi.get_2d_slice(voi.sagital, idx * self.ctx.pixel_size)
                    if slice is not None:
                        data.append(np.array(slice.contour[0].contour))
                        plot = True
                elif self.plot_plan == "Coronal":
                    slice = voi.get_2d_slice(voi.coronal, idx * self.ctx.pixel_size)
                    if slice is not None:
                        data.append(np.array(slice.contour[0].contour))
                        plot = True
                data = self.points_to_plane(data)
                if plot:
                    for d in data:
                        self.figure.plot(d[:, 0], d[:, 1], color=(np.array(voi.get_color()) / 255.0))
        # set zoom
        size = self.get_size()
        width = float(size[0]) / self.zoom * 100.0
        height = float(size[1]) / self.zoom * 100.0
        offset = self.get_offset()
        self.figure.axes.set_xlim(offset[0], offset[0] + width)
        self.figure.axes.set_ylim(offset[1] + height, offset[1])

        self.plot_text(idx)
        self.plot_fields(idx)
        if not self.draw_in_gui:
            self.figure.show()

    def clean_plot(self):
        while len(self.figure.lines) > 0:
            self.figure.lines.pop(0)
        while len(self.figure.texts) > 0:
            self.figure.texts.pop(0)

    def show(self):
        self.figure.show()

    def points_to_plane(self, point):
        d = point
        for data in d:
            if self.plot_plan == "Transversal":
                data[:, 0] /= self.ctx.pixel_size
                data[:, 1] /= self.ctx.pixel_size
            elif self.plot_plan == "Sagital":
                data[:, 0] = (-data[:, 1] + self.ctx.pixel_size * self.ctx.dimx) / self.ctx.pixel_size
                data[:, 1] = (-data[:, 2] + self.ctx.slice_distance * self.ctx.dimz) / self.ctx.slice_distance
            elif self.plot_plan == "Coronal":
                data[:, 0] = (-data[:, 0] + self.ctx.pixel_size * self.ctx.dimy) / self.ctx.pixel_size
                data[:, 1] = (-data[:, 2] + self.ctx.slice_distance * self.ctx.dimz) / self.ctx.slice_distance
        return d

    def plot_fields(self, idx):
        if self.plan is None:
            return
        targets = []
        for v in self.plan.get_vois():
            if v.is_target():
                targets.append(v)
        if len(targets) is 0 or len(targets) > 1:
            return

        target = targets[0].get_voi().get_voi_data()
        # center = target.calculate_center() # TODO why not used ?
        data = None
        if self.plot_plan == "Transversal":
            slice = target.get_slice_at_pos(self.ctx.slice_to_z(idx))
            if slice is not None:
                for contour in slice.contour:
                    data = np.array(contour.contour)
            vec = np.array([0, 0, 1])
        elif self.plot_plan == "Sagital":
            slice = target.get_2d_slice(target.sagital, idx * self.ctx.pixel_size)
            if slice is not None:
                for contour in slice.contour:
                    data = np.array(contour.contour)
            vec = np.array([0, 1, 0])
        elif self.plot_plan == "Coronal":
            slice = target.get_2d_slice(target.coronal, idx * self.ctx.pixel_size)
            if slice is not None:
                for contour in slice.contour:
                    data = np.array(contour.contour)
            vec = np.array([1, 0, 0])

        if data is None:
            return
        for f in self.plan.get_fields():
            if not f.is_selected():
                continue

            gantry = f.get_gantry()
            couch = f.get_couch()
            field_vec = -pytrip.get_basis_from_angles(gantry, couch)[0]

            field_vec = field_vec - np.dot(field_vec, vec) * vec

            if abs(np.linalg.norm(field_vec)) < 0.01:
                continue

            field_vec /= np.linalg.norm(field_vec)
            cross_vec = np.cross(field_vec, vec)
            proj = np.dot(data - slice.calculate_center()[0], np.transpose(cross_vec))
            idx_min = np.where(min(proj) == proj)[0][0]
            idx_max = np.where(max(proj) == proj)[0][0]
            point_min = data[idx_min]
            point_max = data[idx_max]

            steps = int(self.ctx.pixel_size * self.ctx.dimx / 2)
            f1 = np.array([point_min + x * field_vec for x in range(steps)])
            f2 = np.array([point_max + x * field_vec for x in range(steps)])
            plot_data = []

            plot_data.append(f1)
            plot_data.append(f2)
            plot_data = self.points_to_plane(plot_data)

            color = np.array([124, 252, 0]) / 255.0
            self.figure.plot(plot_data[0][:, 0], plot_data[0][:, 1], color=color)
            self.figure.plot(plot_data[1][:, 0], plot_data[1][:, 1], color=color)

    def clear_dose_view(self):
        if hasattr(self, "fig_dose"):
            self.fig_dose.set_visible(False)
            for ax in self.figure.figure.axes:
                if ax is self.dose_bar.ax:
                    self.figure.figure.delaxes(ax)
            del self.fig_dose
        if hasattr(self, "dose_bar"):
            for ax in self.figure.figure.axes:
                if ax is self.dose_bar.ax:
                    self.figure.figure.delaxes(ax)
            del self.dose_bar

    def plot_dose(self, idx):
        if not hasattr(self, "dos"):
            return
        if self.dos is None:
            self.clear_dose_view()
            del self.dos
            return
        if self.plot_plan == "Transversal":
            dos_data = self.dos.cube[idx]
        elif self.plot_plan == "Sagital":
            dos_data = self.dos.cube[-1:0:-1, -1:0:-1, idx]
        elif self.plot_plan == "Coronal":
            dos_data = self.dos.cube[-1:0:-1, idx, -1:0:-1]

        if self.dose_plot == "colorwash":
            if self.dos.target_dose <= 0:
                scale = "rel"
            elif self.dose_axis == "auto" and self.dos.target_dose is not 0.0:
                scale = "abs"
            else:
                scale = self.dose_axis
            if scale == "abs":
                self.factor = 1000 / self.dos.target_dose
            if scale == "rel":
                self.factor = 10
            if hasattr(self, "scale") and not self.scale == scale:
                self.max_dose = np.amax(self.dos.cube) / self.factor
                self.clear_dose_view()
            elif not hasattr(self, "scale"):
                self.max_dose = np.amax(self.dos.cube) / self.factor

            cmap = self.colormap_dose
            cmap._init()
            cmap._lut[:, -1] = 0.7
            cmap._lut[0, -1] = 0.0

            plot_data = dos_data / float(self.factor)
            plot_data[plot_data <= self.min_dose] = self.min_dose
            if not hasattr(self, "fig_dose") or not self.scale == scale:
                self.fig_dose = self.figure.imshow(plot_data, cmap=cmap, vmax=(self.max_dose), aspect=self.aspect)
                if not self.draw_in_gui:
                    pass
                    # TODO why bar is not used ?
                    # bar = self.figure.colorbar()
                else:
                    if not hasattr(self, "dose_bar") and not hasattr(self, "let_bar"):
                        cax = self.figure.figure.add_axes([0.9, 0.1, 0.03, 0.8])
                        self.dose_bar = self.figure.figure.colorbar(self.fig_dose, cax=cax)
                if scale == "abs":
                    self.dose_bar.set_label("Dose (Gy)")
                else:
                    self.dose_bar.set_label("Dose (%)")
            else:
                self.fig_dose.set_data(plot_data)
            self.scale = scale
        elif self.dose_plot == "contour":
            x, y = np.meshgrid(np.arange(len(dos_data[0])), np.arange(len(dos_data)))
            isodose_obj = cntr.Cntr(x, y, dos_data)
            for con in self.dosecontour_levels:
                contour = isodose_obj.trace(con["doselevel"] * 10)
                if len(contour) > 0:
                    self.figure.plot(contour[0][:, 0], contour[0][:, 1], con["color"])

    def plot_text(self, idx):
        size = self.get_size()
        offset = self.get_offset()
        width = size[0]
        height = size[1]
        if self.plot_plan == "Transversal":
            slices = self.ctx.dimz
            slice_dist = self.ctx.slice_distance
        elif self.plot_plan == "Sagital":
            slices = self.ctx.dimy
            slice_dist = self.ctx.pixel_size
        elif self.plot_plan == "Coronal":
            slices = self.ctx.dimx
            slice_dist = self.ctx.pixel_size
        self.figure.text(
            offset[0],
            offset[1] + 3.0 / self.zoom * 100,
            "Slice #: %d/%d\n"
            "Slice Position: %.1f mm" % (idx, slices, idx * slice_dist),
            color="white",
            va="top",
            fontsize=8)
        self.figure.text(
            offset[0] + width / self.zoom * 100,
            offset[1] + 3.0 / self.zoom * 100,
            "W / L: %d / %d" % (self.contrast[1], self.contrast[0]),
            ha="right",
            color="white",
            va="top",
            fontsize=8)
        self.figure.text(
            offset[0],
            offset[1] + (height - 5) / self.zoom * 100,
            self.plot_plan,
            color="white",
            va="bottom",
            fontsize=8)

    def plot_let(self, idx):
        if not hasattr(self, "let"):
            return
        if self.let is None:
            if hasattr(self, "fig_let"):
                self.fig_let.set_visible(False)
                del self.fig_let
                del self.let
            if hasattr(self, "let_bar"):
                for ax in self.figure.figure.axes:
                    if ax is self.let_bar.ax:
                        self.figure.figure.delaxes(ax)
                del self.let_bar
            return

        if self.let_plot == "colorwash":
            cmap = self.colormap_dose
            cmap._init()
            cmap._lut[:, -1] = 0.7
            cmap._lut[0, -1] = 0.0
            if self.plot_plan == "Transversal":
                let_data = self.let.cube[idx]
            elif self.plot_plan == "Sagital":
                let_data = self.let.cube[-1:0:-1, idx, -1:0:-1]
            elif self.plot_plan == "Coronal":
                let_data = self.let.cube[-1:0:-1, -1:0:-1, idx]

            let_data[let_data <= self.min_let] = self.min_let
            if not hasattr(self, "fig_let"):
                self.fig_let = self.figure.imshow(let_data, cmap=cmap, vmax=(self.max_let), aspect=self.aspect)
                if not self.draw_in_gui:
                    pass
                    # TODO why bar is not used ?
                    # bar = self.figure.colorbar()
                else:
                    if not hasattr(self, "let_bar") and not hasattr(self, "dose_bar"):
                        cax = self.figure.figure.add_axes([0.9, 0.1, 0.03, 0.8])
                        self.let_bar = self.figure.figure.colorbar(self.fig_let, cax=cax)
                        self.let_bar.set_label("LET (keV/um)")
            else:
                self.fig_let.set_data(let_data)



import os


def get_files():
    """
    get plans from https://neptun.phys.au.dk/~bassler/TRiP/
    """
    dirname = os.path.join('tests', 'res')
    bname = "TST003"
    return os.path.join(dirname, bname)



"""
    This file is part of PyTRiP.

    PyTRiP is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    PyTRiP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with PyTRiP.  If not, see <http://www.gnu.org/licenses/>
"""
import sys


class AptgType:
    CTX = 0
    DOS = 1


class AptgHeader:
    def __init__(self, data, t):
        self.header_type = t
        self.data = data
        self.set_byte_order()

    def get_header(self):
        output_str = "version " + self.version + "\n"
        output_str += "modality " + self.modality + "\n"
        output_str += "created_by " + self.created_by + "\n"
        output_str += "creation_info " + self.creation_info + "\n"
        output_str += "primary_view " + self.primary_view + "\n"
        output_str += "data_type " + self.data_type + "\n"
        output_str += "num_bytes " + str(self.number_bytes) + "\n"
        output_str += "byte_order " + self.byte_order + "\n"
        output_str += "patient_name " + self.patient_name + "\n"
        output_str += "slice_dimension " + str(self.slice_dimension) + "\n"
        output_str += "pixel_size " + str(self.pixel_size) + "\n"
        output_str += "slice_distance " + str(self.slice_distance) + "\n"
        output_str += "slice_number " + str(self.slice_number) + "\n"
        output_str += "xoffset " + str(self.xoffset) + "\n"
        output_str += "dimx " + str(self.dimx) + "\n"
        output_str += "yoffset " + str(self.yoffset) + "\n"
        output_str += "dimy " + str(self.dimy) + "\n"
        output_str += "zoffset " + str(self.zoffset) + "\n"
        output_str += "dimz " + str(self.dimz) + "\n"
        return output_str

    def get_offset(self):
        data = self.data["images"]
        if self.direction == -1:
            idx = len(data) - 1
        else:
            idx = 0
        return data[idx].ImagePositionPatient

    def check_direction(self):
        data = self.data["images"]
        if data[0].ImagePositionPatient[2] < data[1].ImagePositionPatient[2]:
            self.direction = 1
        else:
            self.direction = -1

    def load_dicom(self):
        self.check_direction()
        if self.header_type == AptgType.DOS:
            data = self.data["rtdose"]
            self.slice_number = data.NumberofFrames
        elif self.header_type == AptgType.CTX:
            data = self.data["images"][0]
            self.slice_number = len(self.data["images"])
        self.created_by = ""  # string.replace(_s," ","-")
        self.creation_info = "created by aptg_export;"
        self.primary_view = "transversal"
        self.version = "1.4"
        self.modality = data.Modality
        self.data_type = "integer"
        self.patient_name = data.PatientsName
        self.number_bytes = data.BitsStored / 8
        self.slice_dimension = data.Rows
        self.pixel_size = data.PixelSpacing[0]
        if data.SliceThickness is '':
            self.slice_distance = self.data["images"][0].SliceThickness
        else:
            self.slice_distance = data.SliceThickness
        pos = self.get_offset()
        self.xoffset = int(pos[0]) / self.pixel_size
        self.yoffset = int(pos[1]) / self.pixel_size
        self.zoffset = int(pos[2]) / self.slice_distance
        self.dimx = data.Rows
        self.dimy = data.Columns
        self.dimz = self.slice_number

    def set_byte_order(self):
        endian = sys.byteorder
        if endian == "little":
            self.byte_order = "vms"
            self.byte_format = "<"
        elif endian == "big":
            self.byte_order = "aix"
            self.byte_format = ">"

    def load_voxel(self, path):
        return



#! /usr/bin/env python
""" Reads .gd files and can convert them into (xmgrace readable) ascii data.

    Can be used in the command line

    Mandatory arguments:
    Name      Type       Default  Description
    filename  STRING     ""       File name of .gd file, should be first.

    Optional arguments:
    Name      Type       Default  Description
    exp       STRING     "exp"    if "exp": Export data of .gd into a .dat file
    agr       STRING     ""       if "agr": Export data of .gd into a .agr file
    LET       STRING     "LET"    if "LET": Export also data entitled "DoseLET"

    Example:
    1) reading the file foo.gd and exporting it as ascii file foo.dat
    python gd2dat.py foo.gd

    2) reading the file foo.gd and exporting it as xmgrace file foo.agr
    python gd2dat.py foo.gd agr
"""

import sys
import os
import string


class ReadGd(object):
    """This class reads .gd files."""

    def __init__(self, filename, exp=False, agr=False, LET=False):
        """ setup of object """
        self.filename = filename
        self.head = []
        self.legend = []
        self.indata = []
        self.xdata = []
        self.data = []
        self.title = ""
        self.xlabel = ""
        self.ylabel = ""
        self.h = 0
        self.export_Bool = exp
        self.agr = agr
        self.LET_Bool = LET

        print('\n# Running the conversion scrip gd2dat.py')
        for par in sys.argv:
            if par == 'agr':
                self.agr = True
                self.export_Bool = True
            elif par == 'LET':
                self.LET_Bool = True
            elif par[:3] == 'exp':
                self.export_Bool = True

        if self.filename is None:
            print("No file name has been specified")
        else:
            print("# Reading the file " + self.filename + " with gd2dat.py")
            self.read()

    def read(self):
        if os.path.isfile(self.filename) is False:
            raise IOError("Could not find file " + self.filename)

        gd_file = open(self.filename, 'r')
        gd_lines = gd_file.readlines()
        gd_file.close()

        line = gd_lines[0]
        line_len = len(line)
        self.title = line[:line_len - 1]

        for line in gd_lines[1:]:
            s = 0
            while s < 10:
                if line[s] == ' ':
                    s += 1
                else:
                    break

            if line[s] == 'x' or line[s] == 'X':
                line_len = len(line)
                self.xlabel = line[s + 2:line_len - 1]

            elif line[s] == 'y' or line[s] == 'Y':
                line_len = len(line)
                self.ylabel = line[s + 2:line_len - 1]
            elif line[s] == 'h' \
                    or line[s] == 'H' \
                    or line[s] == 'a' \
                    or line[s] == 'A':
                line_len = len(line)
                p = 0
                while p < line_len:
                    l2 = line[p:p + 2]
                    legend_Bool = False
                    if l2 == 'x ' or l2 == 'X ':
                        self.head.append('x')
                        self.h += 1
                        self.legend.append("x")
                        legend_Bool = False
                    elif l2 == 'y(' or l2 == 'Y(':
                        self.head.append('y')
                        self.h += 1
                        legend_Bool = True
                    elif l2 == 'm(' or l2 == 'M(':
                        self.head.append('m')
                        self.h += 1
                        legend_Bool = True
                    elif l2 == 'n(' or l2 == 'N(':
                        self.head.append('n')
                        self.h += 1
                        legend_Bool = True
                    elif l2 == 'l(' or l2 == 'L(':
                        self.head.append('l')
                        self.h += 1
                        legend_Bool = True

                    if legend_Bool:
                        legend_Bool = False
                        t = p + 1
                        while t < line_len:
                            if line[t] == ')':
                                self.legend.append(line[p + 2:t])
                                if (self.LET_Bool):
                                    if line[p + 2:t] == "mb^DoseLETme^":
                                        self.head[self.h - 1] = "y"
                                        self.legend[self.h - 1] = "DoseLET"
                                    if line[p + 2:t] == "mb^LETme^":
                                        self.head[self.h - 1] = "y"
                                        self.legend[self.h - 1] = "FluenceLET"

                                p = t
                                break
                            else:
                                t += 1
                    p += 1

            elif line[s] == 'n' or line[s] == 'N':
                break
            elif line[s].isdigit() or line[s] == '-':

                line_len = len(line)
                #                print len(line)     debug line
                #                print line[:line_len-1]    debug line
                linelist = string.split(line[:line_len - 1], ' ')
                # print linelist     debug line
                self.indata.append(linelist)

        num = 0
        while num < self.h:
            ydat = []
            if self.head[num] == "x":
                for ele in self.indata:
                    self.xdata.append(ele[num])
                    ydat.append(float(ele[num]))
                self.data.append(ydat)
            elif self.head[num] == 'y' or self.head[num] == 'm' or self.head[num] == 'n':  # normal data
                for ele in self.indata:
                    ydat.append(float(ele[num]))
                self.data.append(ydat)
            num += 1

        if self.export_Bool:
            self.export()

    def export(self):

        len_file_name = len(self.filename)
        out_file_name = self.filename[:len_file_name - 2]
        if self.agr:
            out_file_name += "agr"
            print('# Writing data in a ".agr" file fragment: ' + out_file_name)
        else:
            out_file_name += "dat"
            print('# Writing data in a ".dat" file: ' + out_file_name)

        out_file = open(out_file_name, 'w')

        if self.LET_Bool:
            print('# Exporting also LET data  ')

        if self.agr:
            header = "# Grace project file\n# \n@version 50122 \n"
            #            header += "@page size 842, 595 \n@page scroll 5% \n"
            header += "@page inout 5% \n@link page off \n"

            str_out = header + '@    title  "' + self.title + ' "\n'

            str_out += '@    xaxis  label "' + self.xlabel
            str_out += ' "\n@    xaxis  label char size 1.500000\n'
            str_out += '@    xaxis  ticklabel char size 1.250000\n'

            str_out += '@    yaxis  label "' + self.ylabel
            str_out += ' "\n@    yaxis  label char size 1.500000\n'
            str_out += '@    yaxis  ticklabel char size 1.250000\n'

            out_file.write(str_out)

        num = -1
        counter = 0

        #        print self.h
        #        print self.head
        #        print self.legend
        #        print "length self.data ", len(self.data)
        #        print "length self.xdata ", len(self.xdata)
        #        exit()

        while num < self.h - 1:

            num += 1
            # The following feature is still under development
            #            title = self.legend[num]
            #            if (title == "Survival" or title == "SpecDose" \
            #                    or title == "PhysDose" ):
            #                print '# Skip data with the title "' + title + '"'
            #                continue
            #            print self.head[num]  # debug line

            if self.head[num] == "x":
                continue

            elif self.head[num] == 'y' or self.head[num] == 'm':  # normal data

                if self.head[num] == 'm':
                    str_hd = self.legend[num]
                    j = 0
                    for sign in str_hd:
                        if sign == '*':
                            break
                        j += 1
                    _legend = str_hd[j + 1:]
                else:
                    _legend = self.legend[num]

                str_out = ' \n'
                if not self.agr:
                    str_out += '# '
                str_out += '@    s' + str(counter) + ' legend  "'
                str_out += _legend + '"\n'
                #                sys.stdout.write(str_out)
                out_file.write(str_out)

                str_out = '@    s' + str(counter) + ' comment "'
                if not self.agr:
                    str_out = '# ' + str_out
                str_out += self.filename + ' "\n'
                for i, ele in enumerate(self.data[num]):
                    str_out += self.xdata[i] + ' ' + str(ele) + '\n'
                # sys.stdout.write(str_out)
                out_file.write(str_out)
                counter += 1

            else:
                continue

            # special handling for 'm'
            #
            # A column with the header 'm' should be multiplied with the
            # column to the left of it (cf GD manual).
            #
            if self.head[num] == 'm':

                str_out = ' \n'
                if not self.agr:
                    str_out += '# '
                _legend = self.legend[num]
                str_out += '@    s' + str(counter) + ' legend  "'
                str_out += _legend + '"\n'
                if not self.agr:
                    str_out += '# '
                str_out += '@    s' + str(counter)
                str_out += ' comment "' + self.filename + ' "\n'
                for i, ele in enumerate(self.indata):
                    str_out += self.xdata[i] + ' ' + str((float(ele[num]) * float(ele[num - 1]))) + '\n'
                # sys.stdout.write(str_out)
                out_file.write(str_out)

                counter += 1

# end special handling for 'm'

if __name__ == '__main__':
    ReadGd(sys.argv[1], LET=True, exp=True)



# from pytrip.paths import *
# import time
# from pytrip.res.point import *
# from pytrip.ctx import *
# from pytrip.vdx import *
# from pytrip.dos import *
# import os
# import gc
# import pytrip.dicomhelper as dh
# import numpy
# import threading
# import matplotlib.pyplot as plt
#
# dicomFolder = '/home/jato/Projects/Oscar/WEPL calculation/data/'
# structureFolder = '/home/jato/Projects/Oscar/WEPL calculation/data/structure'
# gantry = range(180,360,10)
# couch = [0]
#
#
# dcm = dh.read_dicom_folder(structureFolder)
#
# path = os.path.join(dicomFolder,'00')
# c = CtxCube()
# dcm = dh.read_dicom_folder(path)
# c.read_dicom(dcm)
#
# dcm = dh.read_dicom_folder(structureFolder)
# v = VdxCube("",c)
# v.read_dicom(dcm)
#
# gc.collect()
# voi = v.get_voi_by_name("Tumor")
# d = voi.get_voi_cube()
# d.cube = np.array(d.cube,dtype=np.float32)
# voi_cube = DensityProjections(d)
#
# for phase in ['00','10','20','30','40','50','60','70','80','90']:
#     print phase
#     path = os.path.join(dicomFolder,phase)
#     c = CtxCube()
#     dcm = dh.read_dicom_folder(path)
#     c.read_dicom(dcm)
#     p = DensityCube(c)
#     projection = DensityProjections(p)
#     for g in gantry:
#         for c in couch:
#             data,start,basis = projection.calculate_projection(voi,g,c,0,1.0)
#             voi_proj,t1,t2 = voi_cube.calculate_projection(voi,g,c,1,1.0)
#             dmap = data*(voi_proj>0.0)
#             np.save("output/%s-%d-%d"%(phase,g,c),dmap)
#
#
#
#



#!/usr/bin/env python

import sys

from pytrip import CtxCube, dicomhelper


def main(args=sys.argv[1:]):

    if len(args) != 2:
        print("\tusage: dicom2trip folder basename")
        exit()

    basename = args[1]

    # import DICOM
    dcm = dicomhelper.read_dicom_folder(args[0])
    c = CtxCube()
    c.read_dicom(dcm)

    c.write_trip_header(basename + ".hed")
    c.write_trip_data(basename + ".ctx")


if __name__ == '__main__':
    sys.exit(main(sys.argv[1:]))



# from pytrip.paths import *
# import time
# from pytrip.res.point import *
# from pytrip.ctx import *
# from pytrip.vdx import *
# from pytrip.dos import *
# import os
# import gc
# import pytrip.dicomhelper as dh
# import numpy
# import threading
# import matplotlib.pyplot as plt
#
# phases = ['00','10','20','30','40','50','60','70','80','90']
#
# gantry = range(180,360,10)
# couch = [0]
#
# output = np.zeros((len(gantry),len(couch)))
#
# for id1,g in enumerate(gantry):
#     for id2,c in enumerate(couch):
#         for i,phase in enumerate(phases):
#             d = np.load("output/%s-%d-%d.npy"%(phase,g,c))
#             if i == 0:
#                 data = np.zeros((10,d.shape[0],d.shape[1]))
#             data[i,:,:] = d
#         std = np.std(data,0)
#         output[id1,id2] = mean(std)
# plt.imshow(output)
# plt.show()
# np.savetxt("result.txt",output)



"""
    This file is part of PyTRiP.

    PyTRiP is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    PyTRiP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with PyTRiP.  If not, see <http://www.gnu.org/licenses/>
"""


def parse_to_var(data, var, stoptag=""):
    out = {}
    i = 0
    if type(data) is str:
        data = data.split()
    for line in data:
        if line.find(stoptag) > -1:
            break
        items = line.split()
        if items[0] in var:
            if len(items) > 1:
                out[var[items[0]]] = line.replace(items[0] + " ", "")
            else:
                out[var[items[0]]] = True
        i += 1
    return out, i



"""
    This file is part of PyTRiP.

    PyTRiP is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    PyTRiP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with PyTRiP.  If not, see <http://www.gnu.org/licenses/>
"""

import os
import unittest

from pytrip.ctx import CtxCube
from pytrip.vdx import VdxCube
from pytrip.guiutil import PlotUtil

import tests.test_base


class TestCtx(unittest.TestCase):
    def setUp(self):
        testdir = tests.test_base.get_files()
        self.cube000 = os.path.join(testdir, "tst003000.ctx")
        self.vdx000 = os.path.join(testdir, "tst003000.vdx")

    def test_read(self):
        c = CtxCube()
        c.read(self.cube000)
        v = VdxCube(c)
        v.read_vdx(self.vdx000)
        g = PlotUtil()
        g.set_ct(c)
#        g.add_voi(v.get_voi_by_name("ptv"))
#        g.plot(81)
#        g.plot(82)

if __name__ == '__main__':
    unittest.main()



"""
    This file is part of PyTRiP.

    PyTRiP is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    PyTRiP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with PyTRiP.  If not, see <http://www.gnu.org/licenses/>
"""
from pytrip.tripexecuter.pytripobj import pytripObj


class FieldCollection(pytripObj):
    def __init__(self, plan):
        self.save_fields = ["fields"]
        self.plan = plan
        self.fields = []

    def Init(self):
        pass

    def __len__(self):
        return len(self.fields)

    def __iter__(self):
        self.iter = 0
        return self

    def __getitem__(self, i):
        return self.fields[i]

    def next(self):
        if self.iter >= len(self.fields):
            raise StopIteration
        self.iter += 1
        return self.fields[self.iter - 1]

    def add_field(self, field):
        if field.get_name() == "":
            field.name = "Field %d" % (len(self.fields) + 1)
        self.fields.append(field)

    def remove_field(self, field):
        self.fields.remove(field)

    def get_field_by_id(self, field):
        return self.fields[field - 1]

    def get_fields(self):
        return self.fields

    def get_field_by_name(self, name):
        for field in self:
            if field.get_name() == name:
                return field
        return None

    def get_number_of_fields(self):
        return len(self.fields)

    def destroy(self):
        pass



#! /usr/bin/env python
""" convert gd files to xmgrace readable ascii data.
"""

import sys
import os


class ReadGd(object):
    '''read file'''

    def __init__(self, filename):

        if os.path.isfile(filename) is False:
            raise IOError("Could not find file " + filename)

        gd_file = open(filename, 'r')
        gd_lines = gd_file.readlines()
        gd_file.close()
        first = True
        ignore_rest = False
        for line in gd_lines:
            if not (line[0].isdigit()):
                string = "#" + line
                if not first:
                    ignore_rest = True
            else:
                first = False
                if ignore_rest:
                    string = "#" + line
                else:
                    string = line

            sys.stdout.write(string)


if __name__ == '__main__':
    ReadGd(sys.argv[1])



"""
    This file is part of PyTRiP.

    PyTRiP is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    PyTRiP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with PyTRiP.  If not, see <http://www.gnu.org/licenses/>
"""


class pytripObj(object):
    def save(self):
        data = {}
        if hasattr(self, "save_fields"):
            for field in self.save_fields:
                item = getattr(self, field)
                if hasattr(item, "save"):
                    data[field] = item.save()
                elif type(item) is list:
                    if len(item) > 0:
                        data[field] = []
                        for i in item:
                            if hasattr(i, "save"):
                                data[field].append(i.save())
                            else:
                                data[field].append(i)
                elif type(item) is dict:
                    if len(item) > 0:
                        data[field] = {}
                        for i in item.keys():
                            if hasattr(item[i], "save"):
                                data[field][i] = item[i].save()
                            else:
                                data[field] = item
                else:
                    data[field] = item
        return data



"""
    This file is part of PyTRiP.

    PyTRiP is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    PyTRiP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with PyTRiP.  If not, see <http://www.gnu.org/licenses/>
"""
import numpy as np
import dicom
from pytrip.file_parser import parse_to_var


class Rst:
    def __init__(self):
        self.var_dict = {"rstfile": "rstfile",
                         "sistable": "sistable",
                         "patient_id": "patient_name",
                         "projectile": "projectile",
                         "gantryangle": "gantryangle",
                         "couchangle": "couchangle",
                         "#submachines": "submachines",
                         "bolus": "bolus",
                         "ripplefilter": "ripplefilter",
                         "mass": "mass",
                         "charge": "charge",
                         "#particles": "particles"}
        self.machines = []

    def get_submachines(self):
        return self.machines

    def calculate_total_energi(self):
        return

    def load_dicom(self, path):
        self.data = dicom.read_file(path)

    def load_field(self, path):
        with open(path, mode='r') as f:
            data = f.read()
        data = data.split("\n")
        out, i = parse_to_var(data, self.var_dict, "submachine#")
        for key, item in out.iteritems():
            setattr(self, key, item)
        if hasattr(self, "bolus"):
            self.bolus = float(self.bolus)
        for machine in range(int(self.submachines)):
            submachine = SubMachine()
            i = submachine.load_submachine(data, i)
            self.machines.append(submachine)

    def get_stepsize(self):
        if len(self.machines) > 0:
            return self.machines[0].stepsize
        return None

    def get_min_max(self):
        min_x = None
        max_x = None
        min_y = None
        max_y = None
        for submachine in self.machines:
            min_max = submachine.get_raster_min_max()
            if min_x is None or min_x > min_max[0]:
                min_x = min_max[0]
            if max_x is None or max_x > min_max[1]:
                max_x = min_max[1]
            if min_y is None or min_y > min_max[2]:
                min_y = min_max[2]
            if max_y is None or max_y > min_max[3]:
                max_y = min_max[3]
        return [min_x, max_x, min_y, max_y]

    def save_random_error_rst(self, path, sigma):
        # TODO why out here, not being used
        # out = self.generate_random_error_rst(sigma)

        with open(path, "wb+") as fp:
            fp.write("rstfile %s\n" % getattr(self, "rstfile"))
            fp.write("sistable %s\n" % getattr(self, "sistable"))
            fp.write("patient_id %s\n" % getattr(self, "patient_name"))
            fp.write("machine# 0\n")
            fp.write("projectile %s\n" % getattr(self, "projectile"))
            fp.write("charge %s\n" % getattr(self, "charge"))
            fp.write("mass %s\n" % getattr(self, "mass"))
            fp.write("gantryangle %s\n" % getattr(self, "gantryangle"))
            fp.write("couchangle %s\n" % getattr(self, "couchangle"))
            fp.write("stereotacticcoordinates\n")
            fp.write("bolus %.0f\n" % getattr(self, "bolus"))
            fp.write("ripplefilter %s\n" % getattr(self, "ripplefilter"))
            fp.write("#submachines %s\n" % getattr(self, "submachines"))
            fp.write("#particles %s\n" % getattr(self, "particles"))
            for machine in self.machines:
                machine.save_random_error_machine(fp, sigma)

    def generate_random_error_rst(self, sigma):
        out = []
        for submachine in self.machines:
            out.append(submachine.generate_random_error_machine(sigma))


class SubMachine:
    def __init__(self):
        self.raster_points = []

    def get_raster_points(self):
        return self.raster_points

    def get_raster_min_max(self):
        min_x = min(self.raster_points, key=lambda x: x[0])[0]
        min_y = min(self.raster_points, key=lambda x: x[1])[1]
        max_x = max(self.raster_points, key=lambda x: x[0])[0]
        max_y = max(self.raster_points, key=lambda x: x[1])[1]
        return [min_x, max_x, min_y, max_y]

    def get_raster_grid(self):
        min_max = self.get_raster_min_max()
        zero = [-int(min_max[0] / self.stepsize[0]), -int(min_max[2] / self.stepsize[1])]
        size = [(min_max[1] - min_max[0]) / self.stepsize[0] + 1, (min_max[3] - min_max[2]) / self.stepsize[1] + 1]
        grid = np.zeros(size)
        rasterpoints = np.array(self.raster_points)
        grid[np.array(rasterpoints[:, 0] / self.stepsize[0] + zero[0], 'uint8'), np.array(
            rasterpoints[:, 1] / self.stepsize[1] + zero[1], 'uint8')] = rasterpoints[:, 2]
        return grid

    def generate_random_error_machine(self, sigma):
        min_max = self.get_raster_min_max()
        zero = [-int(min_max[0] / self.stepsize[0]), -int(min_max[2] / self.stepsize[1])]
        size = [(min_max[1] - min_max[0]) / self.stepsize[0] + 1, (min_max[3] - min_max[2]) / self.stepsize[1] + 1]
        grid = np.zeros((size))
        rasterpoints = np.array(self.raster_points)
        grid[np.array(rasterpoints[:, 0] / self.stepsize[0] + zero[0], 'uint8'), np.array(
            rasterpoints[:, 1] / self.stepsize[1] + zero[1], 'uint8')] = rasterpoints[:, 2]
        size2 = [size[0] + 4 * int(sigma / self.stepsize[0]), size[1] + 4 * int(sigma / self.stepsize[1])]
        zero2 = [zero[0] + 2 * int(sigma / self.stepsize[0]), zero[1] + 2 * int(sigma / self.stepsize[1])]
        outgrid = np.zeros(size2)
        offset = np.array([[x, y]
                           for x in range(-int(sigma / self.stepsize[0]), int(sigma / self.stepsize[0]) + 1)
                           for y in range(-int(sigma / self.stepsize[1]), int(sigma / self.stepsize[1]) + 1)])
        lengths = (offset[:, 0] * self.stepsize[0]) ** 2 + \
                  (offset[:, 1] * self.stepsize[1]) ** 2
        gauss = np.exp(-0.5 * lengths / sigma**2)
        gauss = gauss / np.sum(gauss)
        offset[:, 0] = offset[:, 0] + 2 * int(sigma / self.stepsize[0])
        offset[:, 1] = offset[:, 1] + 2 * int(sigma / self.stepsize[1])
        for i in range(len(offset)):
            o = offset[i]
            outgrid[o[0]:o[0] + size[0], o[1]:o[1] + size[1]] += gauss[i] * grid
        l = []
        for i in range(len(outgrid)):
            for j in range(len(outgrid[0])):
                if outgrid[i, j] > 2000:
                    l.append([(i - zero2[0]) * self.stepsize[0], (j - zero2[1]) * self.stepsize[1], outgrid[i, j]])
        return l

    def save_random_error_machine(self, fp, sigma):
        rasterpoints = np.array(self.generate_random_error_machine(sigma))
        fp.write("submachine# %d %.2f %d %.1f\n" % (self.idx_energy, self.energy, self.idx_focus, self.focus))
        fp.write("#particles %.5E %.5E %.5E\n" % (np.min(rasterpoints[:, 2]), np.max(rasterpoints[:, 2]),
                                                  np.sum(rasterpoints[:, 2])))
        fp.write("stepsize %.0f %.0f\n" % (self.stepsize[0], self.stepsize[1]))
        fp.write("#points %d\n" % (len(rasterpoints)))
        for point in rasterpoints:
            fp.write("%d %d %.5E\n" % (point[0], point[1], point[2]))

    def load_submachine(self, data, i):
        items = data[i].split()
        self.idx_energy = int(items[1])
        self.energy = float(items[2])
        self.idx_focus = int(items[3])
        self.focus = float(items[4])

        while True:
            i += 1
            line = data[i]
            if line.find("stepsize") > -1:
                items = line.split()
                self.stepsize = [float(items[1]), float(items[2])]
            if line.find("#points") > -1:
                self.points = int(line.split()[1])
                break
            if line.find("#particles") > -1:
                self.min_particles = line.split()[1]
                self.max_particles = line.split()[2]
                self.total_particles = line.split()[3]
        i += 1
        for i in range(i, i + self.points):
            items = data[i].split()
            self.raster_points.append([float(items[0]), float(items[1]), float(items[2])])
        return i + 1



"""
    This file is part of PyTRiP.

    PyTRiP is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    PyTRiP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with PyTRiP.  If not, see <http://www.gnu.org/licenses/>
"""
from pytrip.error import InputError

from pytrip.tripexecuter.pytripobj import pytripObj


class TripVoi(pytripObj):
    def __init__(self, voi):
        self.save_fields = ["name", "target", "max_dose_fraction", "oar", "dose", "hu_offset", "hu_value",
                            "dose_percent", "cube_value"]
        self._voi = voi
        self.name = voi.get_name()
        self.target = False
        self.max_dose_fraction = 100.0
        self.oar = False
        self.dose = 68
        self.hu_offset = None
        self.hu_value = None
        self.selected = False
        self.dvh = None
        self.lvh = None
        self.dose_percent = {}
        self.cube_value = -1

    def set_cube_value(self, value):
        self.cube_value = value

    def get_cube_value(self):
        return self.cube_value

    def get_dvh(self, dose):
        if dose is None:
            return None
        if self.dvh is None:
            (self.dvh, self.min_dose, self.max_dose, self.mean, area) = dose.calculate_dvh(self._voi.get_voi_data())
        return self.dvh

    def set_dose_percent(self, ion, dose_percent):
        if dose_percent == "":
            del self.dose_percent[ion]
        self.dose_percent[ion] = float(dose_percent)

    def get_dose_percent(self, ion):
        if ion in self.dose_percent:
            return self.dose_percent[ion]
        return None

    def get_all_dose_percent(self):
        return self.dose_percent

    def remove_dose_percent(self, ion):
        del self.dose_percent[ion]

    def get_hu_offset(self):
        return self.hu_offset

    def set_hu_offset(self, value):
        if len(value) is 0:
            self.hu_offset = None
            return
        try:
            value = float(value)
            self.hu_offset = value
        except Exception:
            raise InputError("HU Offset should be a number")

    def get_hu_value(self):
        return self.hu_value

    def set_hu_value(self, value):
        if len(value) is 0:
            self.hu_value = None
            return
        try:
            value = float(value)
            self.hu_value = value
        except Exception:
            raise InputError("HU Value should be a number")

    def get_dose(self):
        return self.dose

    def get_max_dose_fraction(self):
        return self.max_dose_fraction

    def get_lvh(self, let):
        if let is None:
            return None
        if self.lvh is None:
            (self.lvh, self.min_let, self.max_let, self.let, area) = let.calculate_lvh(self._voi.get_voi_data())
        return self.lvh

    def clean_cache(self):
        self.dvh = None

        # used for dvh or similar, plan selected

    def is_plan_selected(self):
        return self.selected

    def toogle_plan_selected(self, plan):
        self.selected = not self.selected

    def get_name(self):
        return self.name

    def get_voi(self):
        return self._voi

    def set_max_dose_fraction(self, max_dose_fraction):
        try:
            max_dose_fraction = float(max_dose_fraction)
            if max_dose_fraction < 0:
                raise Exception()
            self.max_dose_fraction = max_dose_fraction
        except Exception:
            raise InputError("Max dose fraction should be " "a number between 0 and 1")

    def set_dose(self, dose):
        try:
            dose = float(dose)
            if dose < 0:
                raise Exception()
            self.dose = dose
        except Exception:
            raise InputError("Dose should be a number larger or equal to 0")

    def toogle_target(self):
        if self.is_oar() is True:
            return False
        if self.target is True:
            self.target = False
        else:
            self.target = True
        return self.target

    def toogle_oar(self):
        if self.is_target() is True:
            return False
        if self.oar is True:
            self.oar = False
        else:
            self.oar = True
        return self.oar

    def is_target(self):
        return self.target

    # used used for 2d plot and similar, global selected
    def is_selected(self):
        return self._voi.is_selected()

    def toogle_selected(self):
        self._voi.toogle_selected()

    def is_oar(self):
        return self.oar



"""
    This file is part of PyTRiP.

    PyTRiP is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    PyTRiP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with PyTRiP.  If not, see <http://www.gnu.org/licenses/>
"""
import os
import csv


class RBEHandler:
    def __init__(self, datafile=""):
        self.datafile = datafile

    def get_rbe_by_name(self, name):
        for rbe in self.get_rbe_list():
            if rbe.get_name() == name:
                return rbe

    def get_rbe_list(self):
        if not hasattr(self, "rbe"):
            self.load_rbe()
        if hasattr(self, "rbe"):
            return self.rbe
        return []

    def load_rbe(self, i=0):
        if os.path.exists(self.datafile):
            with open(self.datafile, "r") as fp:
                reader = csv.reader(fp, delimiter='\t')
                self.rbe = [RBE(x[0], x[1]) for x in reader]
        else:
            if i is 0:
                self.load_rbe_folder()
                self.load_rbe(1)

    def load_rbe_folder(self):
        if not hasattr(self, "rbe_folder") or self.rbe_folder is None:
            return
        path = os.path.expandvars(self.rbe_folder)
        folder = os.listdir(path)
        with open(self.datafile, "w+") as fp_out:
            for item in folder:
                if os.path.splitext(item)[1] == ".rbe":
                    stop = False
                    with open(os.path.join(path, item), "r") as fp:
                        while not stop:
                            line = fp.readline()
                            if line.find("!celltype") > -1:
                                fp_out.write("%s\t%s\n" % (line.split()[1], os.path.join(path, item)))
                                stop = True
                            if not line:
                                stop = True


class RBE:
    def __init__(self, name="", path=""):
        self.name = name
        self.path = path

    def get_name(self):
        return self.name

    def get_path(self):
        return self.path



"""
    This file is part of PyTRiP.

    PyTRiP is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    PyTRiP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with PyTRiP.  If not, see <http://www.gnu.org/licenses/>
"""
# from pytrip.paths import *
# import time
# from pytrip.res.point import *
# from pytrip.ctx import *
# from pytrip.vdx import *
# from pytrip.dos import *
# import pytrip.dicomhelper as dh
# import numpy
# import threading
# #import matplotlib.pyplot as plt

# dcm = dh.read_dicom_folder(
# "../../../DicomData/Til Jacob T anonymiseret/Anonym2")
# c = CtxCube()
# c.read_dicom(dcm)
# ~ c.write_trip_data("../../../DicomData/test/test.ctx")
# ~ c.write_trip_header("../../../DicomData/test/test.hed")

# v = VdxCube("",c)
# v.read_dicom(dcm)
# #~ v.write_to_trip("../../../DicomData/test/test.vdx")
# voi = v.get_voi_by_name("ptv 3mm")
# #~ d.load_from_structure(voi,1000)
# voi.create_point_tree()
# #~ exit()
# p = DensityCube(c)
# projection = DensityProjections(p)
# #~ print(projection.calculate_angle_quality(voi,110,0)
# grid = projection.calculate_quality_grid(voi,range(0,360,2),range(-90,90,2))
# numpy.save("../plotdata",grid)
# exit()

# dcm = dh.read_dicom_folder(
# "../../../DicomData/Til Jacob T anonymiseret/Anonymiseret")
# c = CtxCube()
# c.read_dicom(dcm)
# #~ c.write_trip_data("../../../DicomData/test/test.ctx")
# #~ c.write_trip_header("../../../DicomData/test/test.hed")
#
# v = VdxCube("",c)
# v.read_dicom(dcm)
# #~ v.write_to_trip("../../../DicomData/test/test.vdx")
# voi = v.get_voi_by_name("ctv")
# #~ d.load_from_structure(voi,1000)
# voi.create_point_tree()
# #~ exit()
# p = DensityCube(c)
# projection = DensityProjections(p)
# #~ print(projection.calculate_angle_quality(voi,110,0)
# grid2 = projection.calculate_quality_grid(
# voi,range(0,360,10),range(-90,90,10))
# #~ numpy.save("../plotdata",grid)
#
# plt.imshow(grid/grid2)
# plt.colorbar()
# plt.show()
#
#
#
# #~ print(projection.calculate_angle_quality(voi,130,30)
# #~ for angle in angles:
# gantry = 90
# couch = 0
# print(projection.calculate_angle_quality(voi,gantry,couch))
# exit()
# #~ gantry,couch = angles_from_trip(gantry,couch)
# data,start,basis = projection.calculate_projection(voi,gantry,couch)
# #~ print(voi.calculate_center()
# #~ print(start
# contour = voi.get_2d_projection_on_basis(basis,start)
# #~ gradient = numpy.gradient(data)
# #~ data = (gradient[0]**2+gradient[1]**2)**0.5
# plt.imshow(data>0)
# plt.plot(contour[:,1]/c.pixel_size,contour[:,0]/c.pixel_size,'r')
# plt.colorbar()
# plt.show()
# exit()
#
# #~ print(
# gantry = 100
# couch =  -60
# print(gantry,couch)
# print(projection.calculate_angle_quality(voi,gantry,couch))
#
#
# gantry = 270
# couch =  70
# print(gantry,couch)
# print(projection.calculate_angle_quality(voi,gantry,couch))
#
# gantry = -110
# couch =  10
# print(gantry,couch)
# print(projection.calculate_angle_quality(voi,gantry,couch))
#
# gantry = 270
# couch =  50
# print(gantry,couch)
# print(projection.calculate_angle_quality(voi,gantry,couch))
#
# gantry = 180
# couch =  0
# print(gantry,couch)
# print(projection.calculate_angle_quality(voi,gantry,couch))
#
# gantry = -30
# couch =  60
# print(gantry,couch)
# print(projection.calculate_angle_quality(voi,gantry,couch))
#
# gantry = 100
# couch =  40
# print(gantry,couch)
# print(projection.calculate_angle_quality(voi,gantry,couch))
#
# gantry = 110
# couch =  10
# print(gantry,couch)
# print(projection.calculate_angle_quality(voi,gantry,couch))
#
# gantry = 0
# couch =  -90
# print(gantry,couch)
# print(projection.calculate_angle_quality(voi,gantry,couch))
#
#
# angles = [[100,-60],[270,70],[-110,10],[270,50],[180,0],
# [-30,60],[100,40],[110,10],[0,-90]]
# #~ angles = [[180,90]]
# for angle in angles:
#     data,start,basis = projection.calculate_projection(voi,angle[0],angle[1])
#     contour = voi.get_2d_projection_on_basis(basis,start)
#     gradient = numpy.gradient(data)
#     data = (gradient[0]**2+gradient[1]**2)**0.5
#     #~ print(start
#     #~ print(basis
#     plt.imshow(data,vmax=5.0)
#     plt.colorbar()
#     plt.plot(contour[:,1]/c.pixel_size,contour[:,0]/c.pixel_size,'r')
#     plt.show()
#
